<?php
/**
 * Projekt: WS-Reporter
 *          Alle Methoden zur Erstellung der Daten eines Reports
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    03.10.2022
 *
 */
global $query, $dbDWH, $meldungen, $testMode;
require_once './includes/classes.inc';

if (false) $query = new Query("DUMMY");
$testMode = false;


function clear($query) {
    $query->repRows = new Vector();
    $query->repValues = new Vector();
    $query->repColumns = new Vector();
    $query->repFilter = new Vector();
    $query->repTables = new Vector();
    
    $query->values_in_columns = false;
}

function addMeldung($sql,$text){
    global $meldungen;
    if (strpos($meldungen, $text) === false)
        $meldungen .= $text."\n";
    $sql->sql .= "\n-- ".$text."\n";
}


function setDimUse($mr,$use){
    global $tQuery, $meldungen;    
    if (!array_key_exists($mr->getName(),$tQuery->metaData)) {
//        error_log('setDimUse '.$mr->getName().' ist nicht in '.$tQuery->class_table);
        return;
    }
    $mr = $tQuery->metaData[$mr->getName()];
    $dimNr = 1*$mr->dimNr;
    if ($dimNr >= 0)
        if(array_key_exists($dimNr, $tQuery->dimensions)){
            if($tQuery->dimensions[$dimNr]->use < $use)  $tQuery->dimensions[$dimNr]->use = $use;
        }else $meldungen.="\nFEHLER: Die Dimension Nr ".$dimNr.' gibt es nicht in '.$tQuery->class_table.' aus '.$mr->getName();
    foreach ($mr->reqDim as $dimNr){
        if ($dimNr >= 0)
            if(array_key_exists($dimNr, $tQuery->dimensions)){
                if($tQuery->dimensions[1*$dimNr]->use < $use)  $tQuery->dimensions[1*$dimNr]->use = $use;
        }else $meldungen.="\nFEHLER: Die Dimension Nr ".$dimNr.' gibt es nicht in '.$tQuery->class_table.' aus '.$mr->getName();
    }
}  

/*
 * Eine Zeilen- oder Spalten-Variable ist für eine verknüpfte
 * Tabelle gesperrt, 
 *  * wenn sie dort nicht im Verzeichnis rowVars enthalten ist, oder
 *  * wenn ein explizit gesetzter Filter die Zuordnung in den Zeilen/Spalten
 *    unterdrückt. 
 */
function gesperrteRow($var,$tab){
    global $query, $meldungen, $sperrGrund;
//    error_log('Teste gesperrteRow '.$var.' in '.$tab);
    if ($tab == null || $tab == 'base') {
//        error_log ('tab ist base');
        return false;
    }
    $tJoin = $query->repTables->get($tab);    
    if ($tJoin == null) {
//        error_log('$tJoin ist NULL !!!;');
        return true;
    }
    if ($tJoin->query == null) {
//        error_log('$tJoin->query ist NULL !!!;');
        return true;
    }
    if (! isValid($var)){
//        $meldungen.="FEHLER bei gesperrte Row var=".$var."<br>\n";
//        error_log ('$var is not valid');
        return true;
    }
    if (substr($var,1,1) == ":" || substr($var,1,1) == '=') {
      // für ValuesInColumns
        $var = substr($var,2);
    }
    if ($tJoin->query->rowVars == null){
        echo('$tJoin->query->rowVars == NULL');
        error_log('FEHLER mit ABBRUCH bei gesperrteRow(): $tJoin->query->rowVars == NULL');
        die();
    }
    if (false === array_search($var, $tJoin->query->rowVars)) {
//       $meldungen.=$var .' in '.$tab .' ist in rowVars nicht enthalten und deshalb gesperrt<br>';
        $sperrGrund =$var .' in '.$tab .' ist in rowVars nicht enthalten und deshalb gesperrt';
//        error_log($sperrGrund);
        return true;
    }
//    Stufe ist in der Join-Tabelle durch filter gesperrt
    $tvar = $tJoin->kennzeichen.':'.$var;
    if ($query->repFilter->has($tvar)
        && ! $query->repColumns->has($tvar)){
// ? Warum das??            if ($tJoin->joinType == JoinedTable::JOINOUTERWERTE) return false;
        $sperrGrund =$tvar .' in '.$tab .' ist in repFilter aber nicht in repColumns enthalten und deshalb gesperrt';
//        error_log($sperrGrund);
        return true;
    }
//   $meldungen.=$var .' in '.$tab .' ist nicht gesperrt<br>';
//    error_log($var .' in '.$tab .' ist nicht gesperrt');
    return false;
}


function addRowSelection($mr,$active){
    if ($mr == null) {
        $ev = new Error();
        error_log("FEHLER in addRowSelection: Metadaten sind NULL");
        error_log($ev->getTraceAsString());
        return "\n-- FEHLER in addRowSelection: Metadaten sind NULL\n";
    }
    $sql=",\n";
    if ($active){
        $sql.=$mr->selectVar.' as '.$mr->getSelName();
        if ($mr->hasDispVar()) $sql.=', '.$mr->displayVar.' as '.$mr->getDispName();
        setDimUse($mr, 1);
    }else{
        $sql.='null as '.$mr->getSelName();
        if ($mr->hasDispVar()) $sql.=', null as '.$mr->getDispName();
    }
    return $sql;
}

function addColSelection($mr,$active,$table){
    if ($mr == null) return ",\n-- FEHLER in addColSelection: Metadaten sind NULL\n";
    $sql=",\n";
    if (! isValid($table) || $table == 'base') $table = '';
    else $table .= '__';   
    if ($active){
        $sql.=$mr->selectVar.' as '.$table.$mr->getSelName();
        if ($mr->hasDispVar()) $sql.=', '.$mr->displayVar.' as '.$table.$mr->getDispName();
        if ($mr->dimNr >= 0) setDimUse($mr, 1);
    }else{
        $sql.='null as '.$table.$mr->getSelName();
        if ($mr->hasDispVar()) $sql.=', null as '.$table.$mr->getDispName();
    }
    return $sql;
}

function selectJoinedDispName($mr,$sql){
    global $query;
    if (false) $mr = new MetaRecord();
    $sel = 'base.'.$mr->getDispName();
    for ($i=0; $i < $query->repTables->size();$i++){
        $jt = $query->repTables->getPos($i);
        if ($jt->joinType == JoinedTable::JOINOUTERWERTE){
            if (array_search($mr->getName(),$jt->query->rowVars) !== false)
                $sel = 'ifnull('.$sel.','.$jt->kennzeichen.'.'.$jt->query->metaData[$mr->getName()]->getDispName().')';
        }
    }
    $sql->sql.= ', '.$sel.' as '.$mr->getDispName();    
}
function selectJoinedName($mr,$sql){
    global $query;
    if (false) $mr = new MetaRecord();
    $sel = 'base.'.$mr->getSelName();
    for ($i=0; $i < $query->repTables->size();$i++){
        $jt = $query->repTables->getPos($i);
        if ($jt->joinType == JoinedTable::JOINOUTERWERTE)
            if (array_search($mr->getName(),$jt->query->rowVars) !== false )
                $sel = 'ifnull('.$sel.','.$jt->kennzeichen.'.'.$jt->query->metaData[$mr->getName()]->getSelName().')';
    }
    if (!isValid($mr->displayVar))
        $sql->sql.= ', '.$sel.' as '.$mr->getDispName();
    else
        $sql->sql.= ', '.$sel.' as '.$mr->getSelName();
}

/**
 * Variablen aus der Formel in selectVar extrahieren
 * @param MetaRecord $mr Metarecord des berechneten Werts 
 * @param String $tab Tabellen-alias 'base' oder 'join'. Nur Variablen mit diesem Alias werden geliefert. 
 * @return string[] Liste der Variablen, ggf. mehrfach 
 */
function getCalcBaseVars ($mr,$tab){
    $s = $mr->selectVar;
    $s = str_replace('(', ' ', $s);
    $s = str_replace(')', ' ', $s);
    $s = str_replace('+', ' ', $s);
    $s = str_replace('-', ' ', $s);
    $s = str_replace('/', ' ', $s);
    $s = str_replace('*', ' ', $s);
    $s = str_replace(',', ' ', $s);
    $vb = explode(' ',$s);
    $vl = array();
    foreach ($vb as $v)
        if (substr($v,0,5) == $tab.'.') $vl[] = trim(substr($v,5));
        return $vl;
}

/**
 * Max-Datum für Historisierung vom Typ STICHTAGE
 * bei Dimensionen ermitteln
 * @return string Datum 
 */
function getHistoryQueryDateForDimension($dimension){
    $sql = new SqlAbfrage('DWH');
    $sql->sql = 'select max('.$dimension->histStichtagSelect.') as value '
        .'from '.$dimension->phys_table.' d'.$dimension->dimNr;
    $stmt = $sql->execute();
    $v = $stmt->fetchObject()->value;
    if (isValid($v)) return '"'.$v.'"';
    error_log('getHistoryQueryDateForDimension(): '.$dimension->dimName.' kein Datum über Tabelle gefunden.');
    return 'current_date()';        
}
/**
 * Max-Datum für Historisierung vom Typ STICHTAGE ermitteln.
 * Filter-Definitionen für den Stichtag werden berücksichtigt.
 * @return string Datum 
 */
function getHistoryQueryDate($mrs,$rtKennzeichen){
    global $query, $meldungen;
    if (false) $mrs = new MetaRecord();
    $sql = new SqlAbfrage('DWH');
    if (isValid($mrs->selectFilterTable)){
        if (isValid($rtKennzeichen))
            $meldungen .= '<b>Hinweis:</b> Der Stichtag für die Quelle '.$rtKennzeichen
                ." wird dynamisch über das Maximum in ".$mrs->selectFilterTable." ermittelt.<br>\n";
        else
            $meldungen .= '<b>Hinweis:</b> Der Stichtag für die Basisquelle'
                ." wird dynamisch über das Maximum in ".$mrs->selectFilterTable." ermittelt.<br>\n";
            
        if (substr($mrs->selectFilterTable,0,1) == '#'){
            $mrs->filterSort = Sort::SORTINTERNDESC;
            $mrs->readSelectQuery();
            if (isValid($mrs->getSelectOptions()[0][0])) return '"'.$mrs->getSelectOptions()[0][0].'"';
            error_log('getHistoryQueryDate()1: '.$rtKennzeichen.': '.$mrs->getName().' kein Datum über Auswahlliste gefunden.');
        }else{
            $sql = new SqlAbfrage('DWH');
            $sql->sql = 'select max('.$mrs->getSelectVar().') as value '
                .'from '.$mrs->selectFilterTable;
                if ($query->queryType == Query::DIMENSION) $sql->sql .= ' dim';
                else $sql->sql .= ' base';
                if (isValid($mrs->selectFilterVar)
                    && isValid($rtKennzeichen)
                    && $query->repFilter->get($rtKennzeichen.':'.$mrs->selectFilterVar) != null){
                    $sql->addFilterSQL($mrs->selectFilterVar, $query->repFilter->get($rtKennzeichen.':'.$mrs->selectFilterVar), null);
                }else
                    if (isValid($mrs->selectFilterVar) && $query->repFilter->get($mrs->selectFilterVar) != null){
                        $sql->addFilterSQL($mrs->selectFilterVar, $query->repFilter->get($mrs->selectFilterVar), null);
                }
                
                if (isValid($mrs->selectFilterVar2)
                    && isValid($rtKennzeichen)
                    && $query->repFilter->get($rtKennzeichen.':'.$mrs->selectFilterVar2) != null){
                    $sql->addFilterSQL($mrs->selectFilterVar2, $query->repFilter->get($rtKennzeichen.':'.$mrs->selectFilterVar2), null);
                }else
                    if (isValid($mrs->selectFilterVar2) && $query->repFilter->get($mrs->selectFilterVar2) != null){
                        $sql->addFilterSQL($mrs->selectFilterVar2, $query->repFilter->get($mrs->selectFilterVar2), null);
                }
                $stmt = $sql->executeE(true);
                if ($stmt === false) die();
                $v = $stmt->fetchObject()->value;
                if (isValid($v)) return '"'.$v.'"';
                error_log('getHistoryQueryDate()2: '.$rtKennzeichen.': '.$mrs->getName().' kein Datum über Tabelle gefunden.');
        }
    } else {
        if (isValid($rtKennzeichen))
            $meldungen .= '<b>Hinweis:</b> Der Stichtag für die Quelle '.$rtKennzeichen
                ." wird dynamisch über das Maximum ermittelt.<br>\n";
        else
            $meldungen .= '<b>Hinweis:</b> Der Stichtag für die Basisquelle'
                ." wird dynamisch über das Maximum ermittelt.<br>\n";
                        $sql = new SqlAbfrage('DWH');
        $sql->sql = 'select max('.$mrs->getSelectVar().') as value '
            .'from '.$query->phys_table;
        if ($query->queryType == Query::DIMENSION) $sql->sql .= ' dim';
        else $sql->sql .= ' base';            
        $stmt = $sql->execute();
        $v = $stmt->fetchObject()->value;
        if (isValid($v)) return '"'.$v.'"';
        error_log('getHistoryQueryDate()3: '.$rtKennzeichen.': '.$mrs->getName().' kein Datum über Basis-Tabelle gefunden.');
    }
    
    if (isValid($rtKennzeichen))
        $meldungen .= '<b>Hinweis:</b> Der Stichtag für die Quelle '.$rtKennzeichen
            ." konnte nicht ermittelt werden.<br>\n";
    else
        $meldungen .= '<b>Hinweis:</b> Der Stichtag für die Basisquelle'
            ."  konnte nicht ermittelt werden.<br>\n";
                
    return 'current_date()';
}


// Ende Hilfs-Funktionen
/**
 * Stichtage für Queries und Dimensionen ermitteln.
 * 
 * History-Options
    const HSUEBERNEHMEN = 0;

    const HSSTICHTAGBASE = 2;
    const HSSTICHTAGSELBST = 3;
    
    const HSAKTUELL = 1;
    const HSVJENDE = 4;
    const HSVVJENDE = 5;

    const HSDATUM = 6;
    
Query:
   public $histType = Query::KEINE; // VONBIS, STICHTAG, STICHTAGE
    public $historyOption = 0;
    public $historyDate = null; // wird im Report gesetzt    
    public $historyQueryDate = null; /
    
Dimension:
    public $historyOption = HistoryOptions::HSUEBERNEHMEN;
    public $historyDate = null;
    public $historyQueryDate = null;
    
    public $dimIsFilter = false; // Wenn true wirkt die Dimension auch als Filter, wenn daraus nichts selektiert wurde
    
Die Report-Definition setzt in der Basisquelle
$query->historyOption // Methode zur Ermittlung des Stichtags
$query->historyDate    // ggf. explizit gesetzter Stichtag
und in ihren Dimensionen
$dimension->historyOption // Methode zur Ermittlung des Stichtags
$dimension->historyDate    // ggf. explizit gesetzter Stichtag

das Ergebnis für die SQL-Afrage steht in 
$query->histroyQueryDate
bzw. in 
$dimension->histroyQueryDate
        
        
Bei den verknüpften Quellen wird in der repTable (JoinedTable) mit Arrays gearbeitet:
$rt->dimHistoryOption[] mit $rt->dimHistoryOption[0] für die Datenquelle und $rt->dimHistoryOption[x] für die Dimensionen   
$rt->dimHistoryDate[] mit $rt->dimHistoryDate[0] für die Datenquelle und $rt->dimHistoryDate[x] für die Dimensionen   

historyQueryDate wird in
$rt->dimHistoryQueryDate[] mit $rt->dimHistoryQueryDate[0] für die Datenquelle
gespeichert 
    
 */

function createHistory(){
    global $query, $meldungen;
    if ($query->histType > 0) {
//        error_log('$query->histType='.$query->histType.'  $query->historyOption='.$query->historyOption);
    // historyQueryDate für Basisquelle bestimmen: 
        switch ($query->historyOption) {
            case HistoryOptions::HSUEBERNEHMEN:  // Default - sollte nicht vorkommen
            case HistoryOptions::HSAKTUELL:
                if ($query->histType == Query::STICHTAGE){
                    $mrs = $query->getReportMetaRecord($query->histStichtag);
                    $query->historyQueryDate = getHistoryQueryDate($mrs, null);
                }else $query->historyQueryDate = 'current_date()';
                break;
            ;
            case HistoryOptions::HSOHNEAUSWAHL:
                $query->historyQueryDate = null;
                break;
            case HistoryOptions::HSVJENDE:
                $query->historyQueryDate = 'concat(year(current_date())-1,\'-12-31\')';
                ;
                break;
            case HistoryOptions::HSVVJENDE:
                $query->historyQueryDate = 'concat(year(current_date())-2,\'-12-31\')';
                ;
                break;
            case HistoryOptions::HSSTICHTAGBASE:
            case HistoryOptions::HSSTICHTAGSELBST:
                $mr = $query->metaData[$query->histStichtag];
                if ($mr != null){
                    if (strpos($mr->selectVar,'base.') === false)  $query->historyQueryDate = 'base.'.$mr->selectVar;
                    else  $query->historyQueryDate = $mr->selectVar;
                }
                    else{
                        echo 'FEHLER in CreateHistory 004 Query->Stichtag = '.$query->histStichtag;
                        die();
                        break;
                    }
                ;
                break;
            case HistoryOptions::HSDATUM:
                $query->historyQueryDate = "'".$query->historyDate."'"; // manuelle Eingabe
                ;
                break;
                
            default:
                echo 'FEHLER in CreateHistory 001 histOption = '.$query->historyOption ;
                die();
            break;
        }
        error_log('Basis-Quelle ('.$query->class_table.") hat Hist.-Typ ".$query->histType.'('.Query::getHistText($query->histType)
            .")\n mit Hist.-Option "
            .$query->historyOption.' ('.HistoryOptions::getKlartext($query->historyOption).")\n ergibt Query-Date = {$query->historyQueryDate}");
    }// if histType > 0 der Basisquelle
    
        // Historisierung der Dimensionen
        foreach ($query->dimensions as $dimension){
            if (false) $dimension = new Dimension(null);
            if ($dimension->histType <= 0) continue;;            
            // historyQueryDate für Basisquelle bestimmen:
            switch ($dimension->historyOption) {
                case HistoryOptions::HSUEBERNEHMEN:  
                case HistoryOptions::HSSTICHTAGBASE:
                case HistoryOptions::HSSTICHTAGSELBST:
                    if ($query->historyQueryDate == null){
                        if ($dimension->histType == Query::STICHTAGE){
                            // KEIN MANGEL - sollte nicht oder selten vorkommen
                            $dimension->historyQueryDate = getHistoryQueryDateForDimension($dimension);                                
                            $meldungen .= '<b>Achtung:</b> Der Stichtag für Dim. '.$dimension->dimNr.': '.$dimension->dimName
                            ." wird dynamisch über das Maximum ermittelt.<br>";
                        }else $dimension->historyQueryDate = 'current_date()';                                
                    }else $dimension->historyQueryDate = $query->historyQueryDate;
                    break;
                case HistoryOptions::HSAKTUELL:
                    if ($dimension->histType == Query::STICHTAGE){
                        // MANGEL: Hier müsste der Stichtag auch aus einer Werte-Tabelle selektiert werden. Die ist an dieser Stelle aber unbekannt.
                        $dimension->historyQueryDate = getHistoryQueryDateForDimension($dimension);
                        $meldungen .= '<b>Achtung:</b> Der Stichtag für Dim. '.$dimension->dimNr.': '.$dimension->dimName
                        ." wird dynamisch über das Maximum ermittelt (Mangel).<br>";
                        
                    }else $dimension->historyQueryDate = 'current_date()';
                    ;
                    break;
                case HistoryOptions::HSOHNEAUSWAHL:
                    $dimension->historyQueryDate = null;
                    ;
                    break;
                case HistoryOptions::HSVJENDE:
                    $dimension->historyQueryDate = 'concat(year(current_date())-1,\'-12-31\')';
                    ;
                    break;
                case HistoryOptions::HSVVJENDE:
                    $dimension->historyQueryDate = 'concat(year(current_date())-2,\'-12-31\')';
                    ;
                    break;
                case HistoryOptions::HSDATUM:
                    $dimension->historyQueryDate = "'".$dimension->historyDate."'"; // manuelle Eingabe
                    ;
                    break;
                    
                default:
                    echo 'FEHLER in CreateHistory 002 dimension->histOption = '.$dimension->dimName.' -> '.$dimension->historyOption ;
                    die();
                    break;
            }        
            error_log('Dimension '.$dimension->dimName.' in Basis-Quelle ('.$query->class_table.") hat Hist.-Typ "
                .$dimension->histType.'('.Query::getHistText($dimension->histType)
                .")\n mit Hist.-Option "
                .$dimension->historyOption.' ('.HistoryOptions::getKlartext($dimension->historyOption)
                .")\n ergibt Date = {$dimension->historyQueryDate}");
        } // Ende Dimensionen
    
    // Verknüpfte Quellen
    for ($r = 0; $r < $query->repTables->size(); $r++){
        $rt = $query->repTables->getPos($r);
        if (false) $rt = new JoinedTable(null);
        
        if ($rt->query->histType > 0) {
        // historyQueryDate für Basisquelle bestimmen:
        switch ($rt->dimHistoryOption[0]) {
            case null:
            case '':    
            case 0:
            case HistoryOptions::HSUEBERNEHMEN:  // Default - sollte nicht vorkommen
                error_log('Verknüpfte Quelle '.$rt->kennzeichen.' übernimmt Hist.-Option '.HistoryOptions::getKlartext($query->historyOption)
                .' für hist-Type '.$rt->query->histType.' Option '.$query->historyOption.' Type '.$rt->query->histType);
                switch ($query->historyOption) {                    
                    case HistoryOptions::HSUEBERNEHMEN:  // Default - sollte nicht vorkommen
                    case HistoryOptions::HSAKTUELL:
                        if ($rt->query->histType == Query::STICHTAGE){
                            $mrs = $rt->query->getReportMetaRecord($rt->query->histStichtag);
                            $rt->dimHistoryQueryDate[0] = getHistoryQueryDate($mrs,$rt->kennzeichen);
                            break;
                        }
                    case HistoryOptions::HSVJENDE:
                    case HistoryOptions::HSVVJENDE:
                    case HistoryOptions::HSDATUM:
                    case HistoryOptions::HSOHNEAUSWAHL:
                        $rt->dimHistoryQueryDate[0] = $query->historyQueryDate;                         
                        break;
                    case HistoryOptions::HSSTICHTAGSELBST:
                        if (!isValid($rt->query->histStichtag)){
                            echo 'Hinweis CreateHistory 005 joined Query->Stichtag = '.$rt->kennzeichen.": Stichtag fehlt.\n"; 
                        }else{
                            $mr = $rt->query->metaData[$rt->query->histStichtag];
                            if ($mr != null){
                                if (strpos($mr->selectVar,'base.') === false)  $rt->dimHistoryQueryDate[0] = 'base.'.$mr->selectVar;
                                else $rt->dimHistoryQueryDate[0] = $mr->selectVar;
                            }
                            else{
                                echo 'FEHLER in CreateHistory 005 joined Query->Stichtag = '.$rt->kennzeichen.' -> '.$rt->query->histStichtag;
                                die();
                                break;
                            }
                        }
                            ;
                            break;
                    case HistoryOptions::HSSTICHTAGBASE:
                        if ($rt->joinType == JoinedTable::JOINWERTE
                        ||$rt->joinType == JoinedTable::JOINOUTERWERTE){
                            $rt->dimHistoryQueryDate[0] = '(select max(WSR_Stichtag) from WSR_base)';
                        }
                        else if ($rt->joinType == JoinedTable::JOINSPALTEN){
                            $mr = $rt->query->metaData[$rt->query->histStichtag];
                            if ($mr != null){
                                if (strpos($mr->selectVar,'base.') === false)  $rt->dimHistoryQueryDate[0] = 'base.'.$mr->selectVar;
                                else $rt->dimHistoryQueryDate[0] = $mr->selectVar;
                            }else{
                                echo 'FEHLER in CreateHistory 005a joined Query->Stichtag = '.$rt->kennzeichen.' -> '.$rt->query->histStichtag;
                                die();                                
                            }
                        }
                        break;
                    default:
                        echo 'FEHLER in CreateHistory 003 joined table->histOption = '.$rt->kennzeichen.' -> '.$rt->dimHistoryOption[0] ;
                        die();
                        break;
                }
                break;
            case HistoryOptions::HSAKTUELL:
                if ($rt->query->histType == Query::STICHTAGE){
                    $mrs = $rt->query->getReportMetaRecord($rt->query->histStichtag);
                    if (false) $mrs = new MetaRecord();
                    $rt->dimHistoryQueryDate[0] = getHistoryQueryDate($mrs,$rt->kennzeichen);                    
                }else $rt->dimHistoryQueryDate[0] = 'current_date()';
                ;
                break;
            case HistoryOptions::HSOHNEAUSWAHL:
                $rt->dimHistoryQueryDate[0] = null;
                ;
                break;
            case HistoryOptions::HSVJENDE:
                $rt->dimHistoryQueryDate[0] = 'concat(year(current_date())-1,\'-12-31\')';
                ;
                break;
            case HistoryOptions::HSVVJENDE:
                $rt->dimHistoryQueryDate[0] = 'concat(year(current_date())-2,\'-12-31\')';
                ;
                break;
            case HistoryOptions::HSSTICHTAGSELBST:
                if (isValid($rt->query->histStichtag))
                    $mr = $rt->query->metaData[$rt->query->histStichtag];
                else $mr = null;
                if ($mr != null){
                    if (strpos($mr->selectVar,'base.') === false)  $rt->dimHistoryQueryDate[0] = 'base.'.$mr->selectVar;
                    else  $rt->dimHistoryQueryDate[0] = $mr->selectVar;
                }
                else{
                    echo 'FEHLER in CreateHistory 005 joined table->Stichtag = '.$rt->kennzeichen.' -> '.$rt->query->histStichtag;
                    die();
                    break;
                }
                ;
                break;
            case HistoryOptions::HSSTICHTAGBASE:
                if ($rt->joinType == JoinedTable::JOINWERTE
                        ||$rt->joinType == JoinedTable::JOINOUTERWERTE)
                    $rt->dimHistoryQueryDate[0] = '(select max(WSR_Stichtag) from WSR_base)';
                else if ($rt->joinType == JoinedTable::JOINSPALTEN){
                    $mr = $rt->query->metaData[$rt->query->histStichtag];
                    if ($mr != null){
                        if (strpos($mr->selectVar,'base.') === false)  $rt->dimHistoryQueryDate[0] = 'base.'.$mr->selectVar;
                        else $rt->dimHistoryQueryDate[0] = $mr->selectVar;
                    }else{
                        echo 'FEHLER in CreateHistory 005a joined Query->Stichtag = '.$rt->ennzeichen.' -> '.$rt->query->histStichtag;
                        die();
                    }
                }
                break;
            case HistoryOptions::HSDATUM:
                $rt->dimHistoryQueryDate[0] = "'".$rt->dimHistoryDate[0]."'"; // manuelle Eingabe
                ;
                break;
                
            default:
                echo 'FEHLER in CreateHistory 006 joined table->histOption = '.$rt->kennzeichen.' -> '.$rt->dimHistoryOption[0] ;
                die();
                break;
        }
        error_log('Verknüpfte Quelle '.$rt->kennzeichen.' ('.$rt->query->class_table.") hat Hist.-Typ ".$rt->query->histType
            .'('.Query::getHistText($rt->query->histType).")\n mit Hist.-Option "
            .$rt->dimHistoryOption[0].' ('.HistoryOptions::getKlartext($rt->dimHistoryOption[0]).")\nergibt Query-Date = {$rt->dimHistoryQueryDate[0]}");
    }
        // historyQueryDate für Dimensionen der verknüpften Quelle bestimmen:
        foreach ($rt->query->dimensions as $dimNr => $dimension){
            if (false) $dimension = new Dimension('');
            if ($dimension->histType <= 0) continue;
            // historyQueryDate für Basisquelle bestimmen:
            switch ($rt->dimHistoryOption[$dimNr]) {
                case HistoryOptions::HSUEBERNEHMEN:  // Default - sollte nicht vorkommen
                    if ($rt->dimHistoryQueryDate[0] == null){
                        if ($dimension->histType == Query::STICHTAGE){
                            // KEIN MANGEL: Sollte nicht vorkommen - ist ein Fehlerfall.
                            $rt->dimHistoryQueryDate[$dimNr] = getHistoryQueryDateForDimension($dimension);
                                $meldungen .= '<b>Achtung:</b> Der Stichtag für Dim. '.$rt->kennzeichen.': '.$dimension->dimName
                                ." wird dynamisch über das Maximum ermittelt - sollte nicht vorkommen.<br>";
                                
                        }else $rt->dimHistoryQueryDate[$dimNr] = 'current_date()';
                        
                    }else $rt->dimHistoryQueryDate[$dimNr] = $rt->dimHistoryQueryDate[0];
                    break;
                case HistoryOptions::HSSTICHTAGBASE:
                case HistoryOptions::HSSTICHTAGSELBST:
                    if ($rt->dimHistoryQueryDate[0] == null){
                        if ($dimension->histType == Query::STICHTAGE){
                            // KEIN MANGEL: Kommt selten vor - ist eigentlich ein Fehlerfall.
                            $rt->dimHistoryQueryDate[$dimNr] = getHistoryQueryDateForDimension($dimension);
                                $meldungen .= '<b>Achtung:</b> Der Stichtag für Dim. '.$rt->kennzeichen.': '.$dimension->dimName
                                ." wird dynamisch über das Maximum ermittelt.<br>";
                                
                        }else $rt->dimHistoryQueryDate[$dimNr] = 'current_date()';                                
                    }else $rt->dimHistoryQueryDate[$dimNr] = $rt->dimHistoryQueryDate[0];
                    break;
                case HistoryOptions::HSAKTUELL:
                    if ($dimension->histType == Query::STICHTAGE){
                        // MANGEL: Hier müsste der Stichtag auch aus einer Werte-Tabelle selektiert werden. Die ist an dieser Stelle aber unbekannt.
                        $rt->dimHistoryQueryDate[$dimNr] = getHistoryQueryDateForDimension($dimension);
                            $meldungen .= '<b>Achtung:</b> Der Stichtag für Dim. '.$rt->kennzeichen.': '.$dimension->dimName
                            ." wird dynamisch über das Maximum ermittelt.<br>";
                            
                    }else $rt->dimHistoryQueryDate[$dimNr] = 'current_date()';
                    ;
                    break;
                case HistoryOptions::HSOHNEAUSWAHL:
                    $rt->dimHistoryQueryDate[$dimNr] = null;
                    ;
                    break;
                case HistoryOptions::HSVJENDE:
                    $rt->dimHistoryQueryDate[$dimNr] = 'concat(year(current_date())-1,\'-12-31\')';
                    ;
                    break;
                case HistoryOptions::HSVVJENDE:
                    $rt->dimHistoryQueryDate[$dimNr] = 'concat(year(current_date())-2,\'-12-31\')';
                    ;
                    break;
                case HistoryOptions::HSDATUM:
                    if (isValid($rt->dimHistoryDate[$dimNr]))
                        $rt->dimHistoryQueryDate[$dimNr] = "'".$rt->dimHistoryDate[$dimNr]."'"; // manuelle Eingabe
                        else{
                            $meldungen .= "\n<br>Fehler bei Historisierung: DimNr.:".$dimNr
                                    .' ('.$dimension->dimName.")  manuelles Datum fehlt.";
                            error_log( "Fehler bei Historisierung: DimNr.:".$dimNr
                                    .' ('.$dimension->dimName.") manuelles Datum fehlt.");
                            $rt->dimHistoryQueryDate[$dimNr] = '"1990-01-01"';
                        }
                    ;
                    break;
                    
                default:
                    echo 'FEHLER in CreateHistory 007 Table->dimension->histOption = '.$rt->kennzeichen.' -> '.$dimension->dimName.' -> '.$rt->dimHistoryOption[$dimNr] ;
                    die();
                    break;
            }
            error_log('Dimension Nr. '.$dimNr.' in verknüpfter Quelle '.$rt->kennzeichen.' ('.$rt->query->class_table.") hat Hist.-Typ ".$rt->query->histType
                .'('.Query::getHistText($rt->query->histType).")\n mit Hist.-Option "
                .$rt->dimHistoryOption[$dimNr].' ('.HistoryOptions::getKlartext($rt->dimHistoryOption[$dimNr]).")\nergibt Query-Date = {$rt->dimHistoryQueryDate[$dimNr]}");
        } // Ende Dimensionen
    } // Ende verknüpfte Quellen   
} // Ende createHistory

/**
 * Historisierungs-Option bei Auswahllisten
 * Liefert einen HistoryQueryDate-Ausdruck, der dann in selQuery eingetragen wird.
 * 
 * Annahme?: Wenn, dann sind die Datenquellen bei Auswahllisten von-bis-historisiert.
 *           Das kann aber trügen :-(. 
 *           
 * param: $query : Query 
 *           
 * return String History-Query-Ausdruck
 */
function getHistoryDateForSelection($query){
    switch ($query->historyOption) {
        case HistoryOptions::HSSTICHTAGBASE:
        case HistoryOptions::HSSTICHTAGSELBST:
        case HistoryOptions::HSUEBERNEHMEN:  // Default - sollte nicht vorkommen
        case HistoryOptions::HSAKTUELL:
        case HistoryOptions::HSOHNEAUSWAHL:
            return 'current_date()';
        case HistoryOptions::HSVJENDE:
            return 'concat(year(current_date())-1,\'-12-31\')';
        case HistoryOptions::HSVVJENDE:
            return 'concat(year(current_date())-2,\'-12-31\')';
        case HistoryOptions::HSDATUM:
            if (isValid($query->historyDate)) return "'".$query->historyDate."'"; // manuelle Eingabe
            else error_log('FEHLER in getHistoryDateForSelection 002: query->historyDate fehlt bei HSDATUM');
            break;
        default:
            echo 'FEHLER in getHistoryDateForSelection 001 histOption = '.$query->historyOption ;
    }
    return 'current_date()';
}

/**
 * Tabelle für Auswahlliste erstellen
 * erzeugt die temporäre Tabelle WSR_Auswahlliste
 * @param $selQuery Query
 */

function createSelectionTable($selQuery) {
    global $query;
    $globalQuery = $query;
    $query = $selQuery;
//    if (false) $query = new Query('');
    $table = 'base';
    error_log('Query-Histtype von '.$query->class_table.' ist '.$query->histType.' = '.$query->getKlartextHistoryOption());
    if ($query->histType == Query::VONBIS){
        $hqd = getHistoryDateForSelection($query);
        if ($hqd != null){
            $query->historyQueryDate = $hqd;
            if ($query->dimensions != null && count($query->dimensions) > 0)
                foreach ($query->dimensions as $dimension){
                    $dimension->historyQueryDate = $hqd;
            }
        }
    }else 
        if ($query->histType == Query::STICHTAG){
        if ($query->dimensions != null && count($query->dimensions) > 0)
            foreach ($query->dimensions as $dimension){
            $dimension->historyQueryDate = ' base.'.$query->histStichtag;
        }
    }else
        if ($query->histType == Query::STICHTAGE){
        error_log('FEHLER in createSelectionTable: Historisierung Stichtage ist nicht implementiert.');
    }
            
    $sql = new SqlAbfrage('DWH');
    createBasicQueryS($sql,$table, 1, -1);
    createTable('Auswahlliste', $sql);
//    error_log('Auswahlliste: '.$sql->sql);    
    $query = $globalQuery;
    return true;
}




function createBasicQuery($sql,$table, $stufe, $colGroup) {
    global $query, $user;
    leseRechteVerlangen();

    $maxStufe = $query->repRows->size();
    error_log("createBasicQuery $table RepColumns = ".$query->repColumns->size().' in '.$query->repTables->size());    
    for ($ci=0; $ci < $query->repColumns->size(); $ci++) {
        //echo(" ergab ".strlen($sql));
        for ($i = 0; $i < $maxStufe; $i++) {
            $var = $query->repRows->getPos($i);
            if ($var == null ) {
                echo "<pre>FEHLER VAR=null\n";
                var_dump($query->repRows);
                echo "\n$table $i ?? $maxStufe ".$query->repRows->size().' - '.$stufe.'</pre>';
            }else
            if ($var->summenZeilen) {
                $sql->union();
                error_log("createBasicQuery $table $i CI = ".$ci);
                createBasicQueryS($sql,$table, $i, $ci);
                $sql->unionWeg();
            }
        }
        $sql->union();
        error_log("createBasicQuery $table $maxStufe CI = ".$ci);
        createBasicQueryS($sql,$table, $maxStufe, $ci);
        $sql->unionWeg();
    }
//    if (! $query->values_in_columns) {
        for ($i = 0; $i < $maxStufe; $i++) {
            $var = $query->repRows->getPos($i);
            if ($var->summenZeilen) {
                $sql->union();
                error_log("createBasicQuery $table $i CI = -1");
                createBasicQueryS($sql,$table, $i, -1);
                $sql->unionWeg();                
            }
        }
        $sql->union();
        error_log("createBasicQuery $table $maxStufe CI = -1");
        createBasicQueryS($sql,$table, $maxStufe, -1);
        $sql->unionWeg();
 //   }
   return $sql;
}


function createBasicQueryS($sql,$table, $stufe, $colGroup) {
    global $query, $tQuery, $meldungen, $testMode, $sperrGrund;
    /*
     * Verzeichnis der Variablen in der Select-Klausel um Dubletten zu vermeiden.
     * Hier werden die Namen aus der Report-Definition, also $mr->getName() oder 
     * $table.':'.$mr->getName() gespeichert.
     * Zu Problemen kommt es, wenn die Variable in den Datenquellen mal mit und mal ohne 
     * DispVar deklariert ist. 
     */ 
    $selectVars = array();
    
    // Nur für die Programmierung, damit die Eclipse-Vorschläge funktionieren 
    $mr = new MetaRecord();
    $rc = new RepColumn('');
    $rf = new RepFilter('');
    
    /*
     * Art der Tabelle feststellen. Abhängig davon müssen die Variablen selektiert werden.
     * Bei Join-Werte dürfen keine reinen null-Spalten vorkommen. D.h. NULL as ..
     * nur, wenn die Variable auf einer anderen Stufe verlangt wird.
     * Bei Base und Join-Spalten wird mit Union verknüpft. Deshalb müssen hier alle Variablen
     * in der richtigen Reihenfolge vorkommen.
     */
    // Query-Objekt für aktuelle Tabelle
    $tQuery = $query;
    $tJoin = null;    
    $baseTable = true;
    $joinSpalten = false;
    $joinWerte = false;    
    
    if ($table != null && $table != 'base'){
        $tJoin = $query->repTables->get($table);
        if ($tJoin == null){
            addMeldung($sql,"FEHLER: Kann Join $table nicht finden.");
            return '';
        }
        $tQuery = $tJoin->query;
        if ($tQuery == null){
            addMeldung($sql,"FEHLER: Kann Tabelle $table nicht erstellen.");
            return '';
        }
        if ($query->repTables->get($table)->joinType == JoinedTable::JOINSPALTEN) $joinSpalten = true;
        else $joinWerte = true;
        $baseTable = false;
    }else{
        $table = null;
    }
    

    
    
    // Bei JoinSpalten werden nur die passenden colGruppen generiert
    if ($joinSpalten){
        if ($colGroup < 0 ) return '';
        if ($query->repColumns->size() <= $colGroup){
            addMeldung($sql,"Ungültige ColGroup bei JoinSpalten $colGroup >= ".$query->repColumns->size());
            return '';
        }
        if ($query->repColumns->getPos($colGroup) == null){
            addMeldung($sql,"ColGroup bei JoinSpalten $colGroup ist NULL");
            return '';
        }
        // Spalten-Name muss mit $table.':' beginnen
        if (strlen($query->repColumns->getPos($colGroup)->getName()) <3) return '';
        if (substr($query->repColumns->getPos($colGroup)->getName(),0,2) != $table.':') return ''; 
    }else     // Colgroup mit X:name ist nur bei JoinSpalten gültig
        if ($colGroup >= 0 && substr($query->repColumns->getPos($colGroup)->getName(),1,1) == ':'){
            return '';
    }
    
    
    /*
     * Stufe = -1 ist für Daten-Zeilen ohne Summen
     */
    $maxStufe = $stufe;
    if ($stufe < 0) $maxStufe = $query->repRows->size();
    
    
    // bei $values_in_columns keine Daten für colGroup < 0;
    if ($query->values_in_columns && $colGroup < 0) return '';
    
    // joinWerte nur für Werte, nicht für Spalten
    if (! $query->values_in_columns
        && $joinWerte
        && $colGroup >= 0){
        /**
         * @var RepColumn $rc;
         */            
        $rc = $query->repColumns->getPos($colGroup);
        if (getSourceTable($rc->val_name) != $table) return '';
    }
     
    
        foreach ($query->dimensions as $dim){
            $dim->use = 0;
        }
        
    /*
     * Begin der Select-Anweisung
     */
    
    $sql->sql.="\n-- createQuery Stufe = ".$stufe." colGroup = ".$colGroup."\n";
    
    $sql->sql.="select distinct ".$maxStufe." as Stufe";
    
    if ($colGroup < 0 ) {
        $sql->sql.=", 99 as colGroup";
    }
    else {
        $sql->sql.=", ".$colGroup." as colGroup";
    }
    
    if ($query->histType > Query::KEINE) {      
        if (isValid($query->historyQueryDate)){
            if ($baseTable) $sql->sql .= ', max('.$query->historyQueryDate.") as WSR_Stichtag\n";
            else 
                if ($joinSpalten) $sql->sql .= ", null as WSR_Stichtag\n";
        }
        else if (isValid($query->histStichtag)){
            if ($baseTable) {
                $mr = $query->getReportMetaRecord($query->histStichtag);
                if ($mr != null){
                    $sql->sql .= ', max('.$mr->getSelectVar().") as WSR_Stichtag\n";
                }else{
                    error_log('FEHLER: MetaRecord für Stichtag '.$query->histStichtag.' fehlt');
                    $meldungen.= 'FEHLER: MetaRecord für Stichtag '.$query->histStichtag.' fehlt.<br>';
                    $sql->sql .= ", null as WSR_Stichtag\n";
                }
            }
            else
                if ($joinSpalten) $sql->sql .= ", null as WSR_Stichtag\n";
        }
    }
    /*
     *  Spalten-Variablen und dazugehörige Report-Link-Variablen
     *  Bei verknüpften Werte-Tabellen nur in Verbindung mit values_in_columns
     */ 
//      NEU WEG    if (! $joinWerte || $query->values_in_columns)
//        if ($baseTable) { echo '<pre>';var_dump($query->repColumns);echo '</pre>';}
        for ($i = 0; $i < $query->repColumns->size();$i++) {
            /**
             * @var RepColumn $rc;
             */
            $rc = $query->repColumns->getPos($i);
            
            // Bei wiederholten Spalten wird die Spalte nur die Spalte zur ColGroup verwendet.
            // Dabei muss die Reihenfolge in jeder ColGroup gleich sein, weshalb
            // hier 
/*       2023 WAS SOL DAS????     
            if ($i != $colGroup 
                && $query->repColumns->getPos($colGroup) != null
                && $query->repColumns->getPos($colGroup)->getVarName() == $rc->getVarName())
                $rc = $query->repColumns->getPos($colGroup);
  */          
            $activeVar = false;
            if ($i != $colGroup                
                && ($query->repColumns->getPos($colGroup) == null
                || $query->repColumns->getPos($colGroup)->getVarName() != $rc->getVarName())) $activeVar = false;
            else if ($joinSpalten){
                if (substr($rc->column_name,0,2) == $table.':') $activeVar = true;
            }
            else if (substr($rc->column_name,1,1) == ':'){
                $activeVar = false;
            }
            else if($joinWerte && ! $query->values_in_columns && substr($rc->val_name,0,2) != $table.':'){
                $activeVar = false;
                continue;
            }
            else if (! gesperrteRow($rc->column_name, $table)){
                $activeVar = true;
            }
            
            // column_name muss in jedem Fall selektiert werden
            if (! array_key_exists($rc->getVarName(), $selectVars)
                && (!$joinWerte || 
                    (substr($rc->column_name,1,1) != ':')
                    && ! gesperrteRow($rc->column_name, $table))
                    ){
                $selectVars[$rc->column_name] = 1;
                $mr = $query->getReportMetaRecord($rc->column_name); 
//                $mr = $tQuery->getReportMetaRecord($rc->column_name);
//                if (! array_key_exists($rc->getVarName(), $selectVars)) {
                $selectVars[$rc->getVarName()] = 1;
                    
                if (substr($rc->column_name,1,1) == ':' ) 
                        $sql->sql.= addColSelection($mr, $activeVar,substr($rc->column_name,0,1));
                    else $sql->sql.= addRowSelection($mr, $activeVar);
//               }
            }
            // Die Werte werden erst mit den anderen Werten hinzugefügt. Warum?
            
            // aber ggf. braucht man den Web-Link
            // mögliches Problem bei Join-Spalten ???
            if (isValid($rc->web_link) 
                && substr(trim($rc->web_link),0,1) == '#'){
                $vn = substr(trim($rc->web_link),1);
                if (! array_key_exists($vn, $selectVars)) {
                    $selectVars[$vn] = 1;
                    $mr = $tQuery->getReportMetaRecord($vn);
                    if ($mr != null) $sql->sql.= addRowSelection($mr, true);
                    else if (! $joinWerte){
                        $mr = $query->getReportMetaRecord($vn);
                        if ($mr != null) $sql->sql.= addRowSelection($mr, false);                        
                    }
                }
            }
       }
        // jetzt habe ich alle ColumnVars und web_links aufgelistet
        
    
    /*
     * Zeilen
     * Bei Base und Unions (Spalten) werden alle Zeilen-Variablen benötigt.
     * Bei Joins (Werten) nur die nicht gesperrten.
     */
        
        // Reine Summenzeile, wenn keine Zeilen angefordert
        if ($query->repRows->size() == 0) $sql->sql.=", 1 as S0";
        else
        for ($i=0; $i < $query->repRows->size(); $i++) {
            $var = $query->repRows->getPos($i);
            if (isValid($var->getName())) {
                if ($var->webLinkColumn){
//                    $sql->sql.=",\n ";
                    if ($i < $maxStufe) {
                        if (isValid($var->web_link)
                            && substr($var->web_link,0,1) == "#") {
                                $mro = $tQuery->getReportMetaRecord(substr($var->web_link,1));
                                if ($mro != null) {
                                    // $active = $basetable
                                    $sql->sql.=addRowSelection($mro, true);
                                }
                        }
                    }else {
                        // Summenzeile
                            if (isValid($var->web_link)
                                && substr($var->web_link,0,1) == "#") {
                                    $mro = $tQuery->getReportMetaRecord(substr($var->web_link,1));
                                    if ($mro != null) {
                                        $sql->sql.=addRowSelection($mro, false);
                                    }
                        }
                    }                    
/** if gesperrte row gibt es hier nicht
                else if ($joinSpalten){ // Bei gesperrten Zeilen für JoinSpalten NULL-Werte erzeugen
                    //                        $sql->sql.="-- aus JoinSpalten ".$var->getName()."\n";
                    if ( isValid($var->web_link)
                        && substr($var->web_link,0,1) == "#") {
                            $mro = $tQuery->getReportMetaRecord(substr($var->web_link,1));
                            if ($mro != null) {
                                $sql->sql.=addRowSelection($mro, false);
                            }
                        }
                } // if joinSpalten
*/            
                }else{
                    $sql->sql.=",\n ";                                   
                if ( ! gesperrteRow($var->getName(), $table)){
                    $mr = $tQuery->getReportMetaRecord($var->getName());
                    if ($i < $maxStufe) {
                        $sql->sql.="0 as S".$i;
                        if (! array_key_exists($var->getName(), $selectVars)){
                            $selectVars[$var->getName()] = 1;
//                            $sql->sql.="\n-- aus repRows ".$var->getName()."\n";
                            $sql->sql.=addRowSelection($mr, true);
                            // Zusätzliche Variable für WEB-Link
                            if (isValid($var->web_link)
                                && substr($var->web_link,0,1) == "#") {
                                    $mro = $tQuery->getReportMetaRecord(substr($var->web_link,1));
                                    if ($mro != null) {
                                        // $active = $basetable
                                        $sql->sql.=addRowSelection($mro, true);
                                    }
                                }
                        }
                    }else {
                        // Summenzeile
                        $sql->sql.="1 as S".$i;
                        if (! array_key_exists($var->getName(), $selectVars)){
                            $selectVars[$var->getName()] = 1;
                            $sql->sql.=addRowSelection($mr, false);
                            if (isValid($var->web_link)
                                && substr($var->web_link,0,1) == "#") {
                                    $mro = $tQuery->getReportMetaRecord(substr($var->web_link,1));
                                    if ($mro != null) {
                                        $sql->sql.=addRowSelection($mro, false);
                                    }
                                }
                            }
                        }
                    }else if ($joinSpalten){ // Bei gesperrten Zeilen für JoinSpalten NULL-Werte erzeugen
//                        $sql->sql.="-- aus JoinSpalten ".$var->getName()."\n";
                        if ($i < $maxStufe)
                            $sql->sql.="0 as S".$i;
                        else
                            $sql->sql.="1 as S".$i;
                        if (! array_key_exists($var->getName(), $selectVars)){
                            $selectVars[$var->getName()] = 1;
                            $mr = $query->getReportMetaRecord($var->getName());
                            $sql->sql.=addRowSelection($mr, false);
                            if ( isValid($var->web_link)
                                && substr($var->web_link,0,1) == "#") {
                                    $mro = $tQuery->getReportMetaRecord(substr($var->web_link,1));
                                    if ($mro != null) {
                                        $sql->sql.=addRowSelection($mro, false);
                                    }
                                }
                        }            
                    }else{ // Bei gesperrten Zeilen nur die Stufen-Variablen ausgeben
                        if ($i < $maxStufe) 
                            $sql->sql.="0 as S".$i;
                        else
                            $sql->sql.="1 as S".$i;
                    }                
                } // if isvalid($var->getName()
            } // else $var->webLinkColumn
        }// Schleifen-Ende
        
        /*
         * Hier ggf. Wert-Anzeige für Sonderfilter selektieren
         */
        for ($i=0; $i < $query->repFilter->size(); $i++){
            $rfilter =  $query->repFilter->getPos($i);
            if (false) $rfilter = new RepFilter('');
            if (! $rfilter->isSet()) continue;
            if ($rfilter->isValueFilter()) continue;
            if ($rfilter->isWindowFilter()) continue;
            $vn = $rfilter->getName();
            $mr = $query->getReportMetaRecord($vn);
            if ($mr == null){
                error_log('FEHLER: Metarecord für '.$rfilter->getName().' nicht gefunden');
            }
            if (false) $mr = new MetaRecord();
            if (($mr->filterType == MetaRecord::SONDERFILTER
                || $mr->filterType == MetaRecord::SONDERFILTERSEL)
                && isValid($mr->displayVar)){
                    if (substr($vn,1,1) == ':'){ // Wert aus/für Join-Werte-Tabelle
                        if ($joinWerte && $tJoin->kennzeichen == substr($vn,0,1)){
                            $sql->sql .= ",\n ".$mr->displayVar.' as FD_'.str_replace(':', '_', $vn);
                        }
                    }else if ($baseTable){
                        $sql->sql .= ",\n ".$mr->displayVar.' as FD_'.$vn;
                    }else if ($joinSpalten){
                        $sql->sql .= ",\n null as FD_".$vn;
                    }                
            }
        }
        // Ende Filter-Anzeige        
        
        /*
         * Werte: Verknüpfte Werte werden nur für die jeweils aktuelle Tabelle selektiert
         * Hier müssen wir auf die Variablen für die =-Werte achten, die erst nach dem Join 
         * berechnet werden.
         */
        for ($i=0; $i < $query->repValues->size(); $i++) {
            /**
             * @var RepValue $var
             */
            $var = $query->repValues->getPos($i);
            $vn = $var->getValueName();
            $mr = $query->getReportMetaRecord($vn);
            if ($mr == null) {
                if ($testMode)
                    addMeldung($sql,"FEHLER keine Metdadaten für Spalte ".$i.": ".$vn);
                else    
                addMeldung($sql,"WSReportQuery createQuery FEHLER keine Metdadaten für Spalte ".$i.": ".$vn);
                
            }else 
            if (substr($vn,1,1) == '='){ // wird später berechnet, aber ggf. werden hier die Basiswerte benötigt
                $vl = null;
                if (($baseTable && $var->base == 'base')
                    || (!$baseTable && $var->base == $tJoin->kennzeichen)) $vl = getCalcBaseVars ($mr,'base');
                else if (! $baseTable &&  $tJoin->kennzeichen == substr($vn,0,1))  $vl = getCalcBaseVars ($mr,'join');
                if ($vl != null && count($vl) > 0){
                    foreach ($vl as $vln){
                        if (in_array($vln, $query->rowVars)) $vlnc = 'C_'.$vln;
                        else $vlnc = $vln;
                        if ( ! array_key_exists($vlnc, $selectVars)){                                                                
                            $selectVars[$vlnc] = 1;
                            $mro = $tQuery->getReportMetaRecord($vln);
                            if ($mro == null) {
                                    $jqName = $query->repTables->get(substr($vn,0,1))->query->class_table;
                                    if ($baseTable)
                                        if ($testMode)
                                            addMeldung($sql,"FEHLER keine Metadaten für Spalte  $i ({$vn} für $jqName): base.".$vln);                                            
                                        else
                                        addMeldung($sql,"WSReportQuery createQuery FEHLER keine Metadaten für Spalte  $i ({$vn}): base.".$vln);
                                    else
                                        if ($testMode) 
                                            addMeldung($sql,"FEHLER keine Metadaten für Spalte  $i ({$vn}): join.".$vln);
                                        else
                                        addMeldung($sql,"WSReportQuery createQuery FEHLER keine Metadaten für Spalte  $i ({$vn}): join.".$vln);
                                    if ($testMode) {
//                                        if ($baseTable) echo "\nFEHLER für berechneten Wert: keine Metadaten für Spalte $i (in {$vn}) : base.".$vln."\n";
//                                        else echo "\nFEHLER für berechneten Wert: keine Metadaten für Spalte $i (in {$vn}) : join.".$vln."\n";
                                    }
                                }else
                                if ( (!$var->keineSummen)
                                    || $maxStufe == $query->repRows->size()
                                    ){                                        
                                    $sql->sql.=",\n ";
                                    $sql->sql.=$mro->selectVar.' as '.$mro->getName();
                                }else{
                                    $sql->sql.= ",\n null as ".$mro->getName();
                                }
                        }
                    }
                }else if ($joinSpalten){
                    $vl = getCalcBaseVars ($mr,'base');
                    if ($vl != null && count($vl) > 0){
                        foreach ($vl as $vln){
                            if (in_array($vln, $query->rowVars)) $vlnc = 'C_'.$vln;
                            else $vlnc = $vln;
                            if ( ! array_key_exists($vlnc, $selectVars)){
                                $selectVars[$vlnc] = 1;                                            
                                $mro = $query->getReportMetaRecord($vln);
                                if ($mro == null) {
                                        if ($testMode) addMeldung($sql,"FEHLER keine Metadaten für Spalte $i ({$vn}) : ".$vln);
                                        else addMeldung($sql,"WSReportQuery createQuery FEHLER keine Metadaten für Spalte  $i ({$vn}): ".$vln);
                                    }
                                else  $sql->sql.=",\n null as".$mro->getName();
                                }
                        }
                    }
                }
            }// =-Values
            else if (substr($vn,1,1) == ':'){ // Wert aus/für Join-Werte-Tabelle
                if ($joinWerte && $tJoin->kennzeichen == substr($vn,0,1)){
                    if ( ! array_key_exists($mr->getName(),  $selectVars)){
                        $selectVars[$mr->getName()] = 1;
                        if ( (!$var->keineSummen)  
                            || $maxStufe == $query->repRows->size()){
                            $sql->sql.=",\n ";
                            $sql->sql.=$mr->selectVar;
                            $sql->sql.=" as ";
//                            $sql->sql.=$mr->getSelName();
                        }else{
                            $sql->sql.= ",\n null as "; //.$mr->getName();
                        }
                        $sql->sql .= $mr->getName();
                    }
                }
            }else if ($baseTable){
                if ( ! array_key_exists($mr->getName(), $selectVars)){
                    $selectVars[$mr->getName()] = 1;
                    if ( (!$var->keineSummen)
                        || $maxStufe == $query->repRows->size()){
                            $sql->sql.=",\n ";
                            $sql->sql.=$mr->selectVar;
                            $sql->sql.=" as ";
//                            $sql->sql.=$mr->getSelName();
                    }else{
                        $sql->sql.= ",\n null as "; // .$mr->getName();
                    }
                    $sql->sql .= $mr->getName();
                }
            }else if ($joinSpalten ){
                if ( ! array_key_exists($mr->getName(),  $selectVars)){
                    $selectVars[$mr->getName()] = 1;
                    $sql->sql.=",\n null as ".$mr->getName();
                }
            }
        }  // ENDE for $query-repValuesSize
       
        /*
         * Werte für Spalten. 
         * Hier können nur Werte aus der Basistabelle oder einer $joinSpalten-Tabelle
         * vorkommen.
         */
        for ($i=0; $i < $query->repColumns->size(); $i++) {
            error_log($query->repColumns->getPos($i)->getName().' - VN: '.$query->repColumns->getPos($i)->getVarName().' - Val: '.
                $query->repColumns->getPos($i)->val_name.' - Table '.$table.': vn=');
            if (! $joinWerte 
                ||substr($query->repColumns->getPos($i)->val_name,0,2) == $table.':'
                ||substr($query->repColumns->getPos($i)->val_name,0,2) == $table.'='
                ){
                $rc = $query->repColumns->getPos($i);
            $vn = $rc->val_name;
            $mr = $query->getReportMetaRecord($vn);
            if ($joinWerte ) $vns = substr($vn,2);
            
            else {
                if (($baseTable || $joinSpalten) &&
                    substr($query->repColumns->getPos($i)->getVarName(),1,1) != ':' &&
                    (substr($query->repColumns->getPos($i)->val_name,1,1) == ':'
                        ||substr($query->repColumns->getPos($i)->val_name,1,1) == '=')
                    ) continue;
                $vns = $vn;
            }
                
            if (isValid($vn) && ! array_key_exists($vns, $selectVars)) {
                $selectVars[$vns] = 1;
                if ($mr == null) {
                    if ($testMode) addMeldung($sql,"FEHLER keine Metadaten für Spalte $i : ".$vn);
                    else addMeldung($sql,"WSReportQuery createQuery FEHLER keine Metadaten für Spalte ".$i.": ".$vn);
                    //                return '';
                }else {
                    error_log('select '.$vn.$mr->getSelectVar());
                    if (substr($vn,1,1) == '=' && ! $joinWerte){ // darf nicht vorkommen, weil base und $joinSpalten-Tabelle über UNION verknüpft werden
                        addMeldung($sql,"WSReportQuery createQuery FEHLER Spalte".$i.": ".$vn.' In den Spalten können keine Werte berechnet werden.');
                    }
                    else if (substr($vn,1,1) == ':'){ // Wert aus Join-Werte-Tabelle
                        $sql->sql.=",\n ";
                            if ($tJoin != null && $tJoin->kennzeichen == substr($vn,0,1)
                                && ((!$rc->keineSummen)
                                    || $maxStufe == $query->repRows->size())
                                )
                                $sql->sql.=$mr->selectVar;
                            else $sql->sql.= ' null ';
                            $sql->sql.= " as ";
                            if ($joinWerte && ! $query->values_in_columns) $sql->sql.= $mr->getSelName();
                            else $sql->sql.= substr($vn,0,1).'__'.$mr->getSelName();
                        
                    }else if ($baseTable){
                        if ( (!$rc->keineSummen)
                            || $maxStufe == $query->repRows->size()
                            ){
                                $sql->sql.= ",\n ";
                                $sql->sql.= $mr->selectVar;
                                $sql->sql.= " as ";
                                $sql->sql.= $mr->getSelName();
                        }else{
                            $sql->sql.= ",\n null as ".$mr->getSelName();
                        }
                        
                    }else if ($joinSpalten){
                        $sql->sql.=",\n null as ";
                        $sql->sql.=$mr->getSelName();
                    }
                }
            }
        } // ENDE Werte für repColumns
        }//TESTER
/**
 * ValueFilter werden erst bei selectSortedTable wirksam.
 * Hier müssen ggf. die Werte zu ihrer Berechnung selektiert werden.
 * Aus verschiedenen Quellen berechnete Filter funktionieren nur, wenn der Wert auch angezeigt
 * wird. Deshalb muss bei Feldern vom Typ Q=Filter hier nichts gemacht werden.
 */        
        if ($query->repFilter->size() > 0) {
            for ($i=0; $i < $query->repFilter->size(); $i++) {
                $var = $query->repFilter->getPos($i);
                if (! $var->isValueFilter()) continue;
                $vn = $var->getName();
                if (substr($vn, 1,1) == '=') continue;
                $mr = $query->getReportMetaRecord($vn);
                if ($mr == null) {
                    if ($testMode) addMeldung($sql,"FEHLER keine Metdadaten für Spalte ".$i.": ".$var->getName());
                    else addMeldung($sql,"WSReportQuery createQuery FEHLER keine Metdadaten für Spalte ".$i.": ".$var->getName());
                    return null;
                }
                
                
                if (substr($vn,1,1) == ':'){ // Wert aus/für Join-Werte-Tabelle
                    if ($joinWerte && $tJoin->kennzeichen == substr($vn,0,1)){
                        if ( ! array_key_exists($mr->getName(),  $selectVars)){
                            $selectVars[$mr->getName()] = 1;
                                    $sql->sql.=",\n ";
                                    $sql->sql.=$mr->selectVar;
                                    $sql->sql.=" as ";
                                    $sql->sql.=$mr->getSelName();
                        }
                    }
                }else if ($baseTable){
                    if ( ! array_key_exists($mr->getName(), $selectVars)){
                        $selectVars[$mr->getName()] = 1;
                                $sql->sql.=",\n ";
                                $sql->sql.=$mr->selectVar;
                                $sql->sql.=" as ";
                                $sql->sql.=$mr->getSelName();
                    }
                }
            }          
        }
        $sql->sql.="\n from ";
        $sql->sql.= $tQuery->phys_table;
        if ($query->queryType == Query::DIMENSION) {
            error_log('Query für Dimension');
            $sql->sql.= " dim";
        }
        else  $sql->sql.= " base";

        rechteAnwenden();
/**
 * Erforderliche Dimensionen für Filter und Werte bestimmen
 * 
 * AKTUELL MIT FEHLER!!!???
 */
        
        if ($query->repFilter->size() > 0) {
            for ($i=0; $i < $query->repFilter->size(); $i++) {
                $var = $query->repFilter->getPos($i);
                if ($var->isWindowFilter()) continue;                
                $var->erledigt = false;
                $vn = $var->getName();
                if ($baseTable && substr($vn, 1,1) == ':') continue;
                if ($var->isValueFilter()) continue;
                if (!$var->isSet()) continue;                    
                $mr = $query->getReportMetaRecord($vn);
                if (! $baseTable && substr($vn, 1,1) != ':') $mr = $tQuery->getReportMetaRecord($vn);              
                if ($mr == null) continue;
                if (($baseTable || $joinSpalten) && $mr->baseFilter()) {
                    continue;
                }
                setDimUse($mr, 2);                
            }
        }
        
        if ($colGroup >= 0){
                $var = $query->repColumns->getPos($colGroup);                
                $vn = $var->filter;
                if (isValid($vn)){
                    $mr = $query->getReportMetaRecord($vn);
                    if ($mr != null){
                        setDimUse($mr, 2);                        
                    }
                }
            }
        
        

        if ($query->repValues->size() > 0) {
            for ($i=0; $i < $query->repValues->size(); $i++) {
                $var = $query->repValues->getPos($i);
                $vn = $var->getValueName();   
                if ($baseTable && substr($vn, 1,1) == ':') continue;
                if ($joinWerte && substr($vn, 0,2) != $table.':') continue;
                $mr = $query->getReportMetaRecord($vn);
                if ($mr == null){
                    error_log("Fehler by repValues $i: $vn Keine Metadaten bei {$query->class_table} ({$tQuery->class_table})");
                } else
                setDimUse($mr, 1);
            }
        }
        
        $dimVrz = array();
        
//        error_log('Dimensions-Joins für '.$table.' colGroup='.$colGroup.' Stufe='.$stufe);
        foreach ($tQuery->dimensions as $dimNr => $dim){
            if (false) $dim = new Dimension('M');
            if (!isset($dim->dimIsFilter) && $stufe < 2 && $baseTable) 
                error_log('DimIsFilter fehlt '.$dimNr.' -> '.print_r($dim,true));
            if ($dim->dimIsFilter) $dim->use = 2;
            if ($dim->use == 2){
//                error_log("Dimension $dimNr:  ".$dim->dimName);
                
                $dimVrz[$dimNr] = true;
                $mr = $tQuery->getReportMetaRecord($dim->baseKey);
                if ($mr != null){
                    $sql->sql .= "\n join ";
                    $sql->sql .= $dim->phys_table.' d'.$dim->dimNr;
                    $sql->sql .= ' on '.$dim->dimSelectVar;                
                    $sql->sql .= ' = '.$mr->getSelectVar();
  // if ($baseTable)  error_log(print_r($dim,true));
                    if ($baseTable){
                        if ($dim->histType == Query::VONBIS){
                            if (isValid($dim->historyQueryDate)){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$dim->historyQueryDate; 
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$dim->historyQueryDate;
                            }else if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';                                
                            }
                        }else
                            if ($dim->histType == Query::STICHTAG
                                || $dim->histType == Query::STICHTAGE){
                                if (isValid($dim->historyQueryDate)){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$dim->historyQueryDate;
                                } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                                }
                        }
                    }else if ($joinWerte){
                        if ($dim->histType == Query::VONBIS){
                           if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];                        
                           } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                               $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                               $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
                           }                            
                        }else 
                            if ($dim->histType == Query::STICHTAG
                                || $dim->histType == Query::STICHTAGE){
                                if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                    && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                        $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                                }
                        }                    
                    }else if ($joinSpalten){
                        if ($dim->histType == Query::VONBIS){
                            if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                    $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                    $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                            } else  if ($tJoin->dimHistoryOption[$dim->dimNr] != HistoryOptions::HSOHNEAUSWAHL){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
                            }
                        }else
                            if ($dim->histType == Query::STICHTAG
                                || $dim->histType == Query::STICHTAGE){
                                if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                    && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                        $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                } else  if ($tJoin->dimHistoryOption[$dim->dimNr] != HistoryOptions::HSOHNEAUSWAHL){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                                }
                        }
                    }else{
                        error_log("Typ 2 Dimension $dimNr {$dim->dimName} passt nicht zu {$tQuery->class_table} mit Key {$dim->baseKey}");
                        $messages .= "\nFEHLER: Typ 2 Dimension $dimNr {$dim->dimName} passt nicht zu {$tQuery->class_table} mit Key {$dim->baseKey}";
                        continue;
                    }
/**  Hier startet der Versuch  Mögliche Filter in der ON-Klausel unterbringen */
                    if (false) {
                    $sql->firstWhere = false;
                    
                    for ($rf=0; $rf < $query->repFilter->size(); $rf ++){
                        $var = $query->repFilter->getPos($rf);
                        if ($var->erledigt) continue;
                        
                        $vn = $var->getName();
                        if ($var->isWindowFilter()) {
                            continue;
                        }
                        if ($var->isValueFilter()) {
                            continue;
                        }
                        $mr = $query->getReportMetaRecord($vn);
                        if ($mr == null) continue;
                        
                        if (($baseTable || $joinSpalten) && $mr->baseFilter()) continue;
                        if ($mr == null) {
                            addMeldung($sql,"createReport FEHLER keine Metdadaten für Spalte ".$i.": ".$var->getName());
                            return null;
                        }
                        // Prüfung ob alle Dimensionen vorhanden sind
                        if ($mr->dimNr < 0  
                            || array_key_exists($mr->dimNr, $dimVrz)) {
                                $alleDimensionenSindDa = true;
                            }
                            else {
                                if ($baseTable) error_log('BaseTable WHDIM: '.$mr->dimNr.' '.$mr->getName().' MR-Dim FEHLT ');
                                else error_log($tJoin->kennzeichen.' BaseTable WHDIM: '.$mr->dimNr.' '.$mr->getName().' MR-Dim FEHLT ');
                                continue;
                            }
                        
                        foreach ($mr->reqDim as $rDim){
                            if (! array_key_exists($rDim, $dimVrz)) {
                                $alleDimensionenSindDa = false;
                                if ($baseTable) error_log('BaseTable WHDIM: '.$dimNr.' '.$mr->getName().' Dim FEHLT ');
                                else error_log($tJoin->kennzeichen.' BaseTable WHDIM: '.$dimNr.' '.$mr->getName().' Dim FEHLT ');
                                continue;
                            }
                        }
                        
                        if ($baseTable) error_log('BaseTable WHDIM: '.$dimNr.' '.$mr->getName().' kann '.$alleDimensionenSindDa);
                        else error_log($tJoin->kennzeichen.' BaseTable WHDIM: '.$dimNr.' '.$mr->getName().' kann '.$alleDimensionenSindDa);
                            
                        if ($alleDimensionenSindDa) $var->erledigt = true;
                        else continue;
                            
                        // Werte-Filter wirken erst bei selectSortedTable()
                        if (isValid($vn)
                            && $var->abSpalte <= $maxStufe ) {
                                if ($baseTable){
                                    //                           if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                    if(substr($vn,1,1) != '=' && substr($vn,1,1) != ':'){
                                        //                                if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                        $sql->addFilterSQL($var->getName(), $var, null);
                                    }
                                }else
                                    if(substr($vn,1,1) == ':' && substr($vn,0,1) == $tJoin->kennzeichen){
                                        if ($var->isSet()){
                                            //                                        if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                            $sql->addFilterSQL($var->getName(), $var, $tJoin);
                                        }
                                        /* leere Filter blockieren und werden nicht mehr gefüllt
                                         else{
                                         $bFilterName = substr($vn,2);
                                         if ($query->repFilter->has($bFilterName)){
                                         if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                         $bFilter = $query->repFilter->get($bFilterName);
                                         $sql->addFilterSQL($vn, $bFilter, null);
                                         }
                                         }
                                         */
                                }else
                                    if(substr($vn,1,1) != ':'
                                        && array_key_exists($vn, $tQuery->metaData)
                                        && ! $query->repFilter->has($tJoin->kennzeichen.':'.$vn)
                                        ){
                                            $mr = $tQuery->getReportMetaRecord($vn);
                                            if ($mr->filterType != MetaRecord::NONE){
                                                if ($var->isSet()){
                                                    //                                            if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                                    $sql->addFilterSQL($tJoin->kennzeichen.':'.$vn, $var, $tJoin);
                                                }
                                            }
                                }
                            }else {
                                // ("-- Übergehe Filter ".$key." bei Stufe ".$maxStufe);
                            }
                    } // ENDER for $rf-Schleife
                    $sql->firstWhere = true;
}
/* ENDE des Versuchs, die Where-Klauseln durch ON-Klauseln zu ersetzen  */                    
                }else{
                    $meldungen .= "\nFEHLER bei Dimension Nr. $dimNr : MetaRecord für Base-Key ".$dim->baseKey.' fehlt';
                    error_log("FEHLER bei Dimension Nr. $dimNr : MetaRecord für Base-Key ".$dim->baseKey.' fehlt');
                }
        }
        }
        
        foreach ($tQuery->dimensions as $dimNr => $dim){
            if ($dim->use  == 1) {
                $mr = $tQuery->getReportMetaRecord($dim->baseKey);
                if ($mr != null){
                    $sql->sql .= "\n left join ";
                    $sql->sql .= $dim->phys_table.' d'.$dim->dimNr;
                    $sql->sql .= ' on '.$dim->dimSelectVar;
                    $sql->sql .= ' = '.$mr->getSelectVar();
                    if ($baseTable){
                        if ($dim->histType == Query::VONBIS){
                            if (isValid($dim->historyQueryDate)){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$dim->historyQueryDate;
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$dim->historyQueryDate;
                            } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
                            }
                        }else
                            if ($dim->histType == Query::STICHTAG
                                || $dim->histType == Query::STICHTAGE){
                                if (isValid($dim->historyQueryDate)){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$dim->historyQueryDate;
                                } else  if ($dim->historyOption != HistoryOptions::HSOHNEAUSWAHL){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                                }
                        }
                    }else if ($joinWerte){
                        if ($dim->histType == Query::VONBIS){
                            if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                    $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                    $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                            } else  if ($tJoin->dimHistoryOption[$dim->dimNr] != HistoryOptions::HSOHNEAUSWAHL){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
                            }
                        }else
                            if ($dim->histType == Query::STICHTAG
                                || $dim->histType == Query::STICHTAGE){
                                if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                    && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                        $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                } else  if ($tJoin->dimHistoryOption[$dim->dimNr] != HistoryOptions::HSOHNEAUSWAHL){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                                }
                        }
                    }else if ($joinSpalten){
                        if ($dim->histType == Query::VONBIS){
                            if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                    $sql->sql .= "\n and ".$dim->histAbSelect.' <= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                    $sql->sql .= "\n and ".$dim->histBisSelect.' >= '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                            } else  if ($tJoin->dimHistoryOption[$dim->dimNr] != HistoryOptions::HSOHNEAUSWAHL){
                                $sql->sql .= "\n and ".$dim->histAbSelect.' <= current_date()';
                                $sql->sql .= "\n and ".$dim->histBisSelect.' >=  current_date()';
                            }
                        }else
                            if ($dim->histType == Query::STICHTAG
                                || $dim->histType == Query::STICHTAGE){
                                if (array_key_exists($dim->dimNr, $tJoin->dimHistoryQueryDate)
                                    && isValid($tJoin->dimHistoryQueryDate[$dim->dimNr])){
                                        $sql->sql .= "\n and ".$dim->histStichtagSelect.' = '.$tJoin->dimHistoryQueryDate[$dim->dimNr];
                                } else  if ($tJoin->dimHistoryOption[$dim->dimNr] != HistoryOptions::HSOHNEAUSWAHL){
                                    $sql->sql .= "\n and ".$dim->histStichtagSelect.' = current_date()';
                                }
                        }
                    }else{
                         error_log("Typ 1 Dimension $dimNr {$dim->dimName} passt nicht zu {$tQuery->class_table} mit Key {$dim->baseKey}");
                    }
            }   
        }
        }
        
        
/**
 * Filter auf der Ebene Basis-Tabelle
 */        
        if ($query->repFilter->size() > 0) {
            for ($i=0; $i < $query->repFilter->size(); $i++) {
                $var = $query->repFilter->getPos($i);
                if ($var->erledigt) continue;
                $vn = $var->getName();
                if ($var->isWindowFilter()) {
                    continue;
                }
                if ($var->isValueFilter()) {
                    continue;
                }
                $mr = $query->getReportMetaRecord($vn);
                if ($mr == null) continue;
                if (($baseTable || $joinSpalten) && $mr->baseFilter()) continue;
                if ($mr == null) {
                    addMeldung($sql,"createReport FEHLER keine Metdadaten für Spalte ".$i.": ".$var->getName());
                    return null;
                }
                // Werte-Filter wirken erst bei selectSortedTable()
                if (isValid($vn)                    
                    && $var->abSpalte <= $maxStufe ) {
                        if ($baseTable){
 //                           if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                            if(substr($vn,1,1) != '=' && substr($vn,1,1) != ':'){      
//                                if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                $sql->addFilterSQL($var->getName(), $var, null);
                            }
                        }else
                            if(substr($vn,1,1) == ':' && substr($vn,0,1) == $tJoin->kennzeichen){
                                if ($var->isSet()){
//                                        if ($testMode) $sql->sql.="\n\n-- Filter: $vn";                                        
                                        $sql->addFilterSQL($var->getName(), $var, $tJoin);
                                }
/* leere Filter blockieren und werden nicht mehr gefüllt                                
                                else{
                                    $bFilterName = substr($vn,2);
                                    if ($query->repFilter->has($bFilterName)){
                                        if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                        $bFilter = $query->repFilter->get($bFilterName);
                                        $sql->addFilterSQL($vn, $bFilter, null);
                                    }
                                }
                                  */  
                        }else
                            if(substr($vn,1,1) != ':' 
                                && array_key_exists($vn, $tQuery->metaData)   
                                && ! $query->repFilter->has($tJoin->kennzeichen.':'.$vn)
                                ){
                                $mr = $tQuery->getReportMetaRecord($vn);
                                if ($mr->filterType != MetaRecord::NONE){
                                    if ($var->isSet()){
//                                            if ($testMode) $sql->sql.="\n\n-- Filter: $vn";
                                            $sql->addFilterSQL($tJoin->kennzeichen.':'.$vn, $var, $tJoin);
                                    }
                                }
                        }
                        
                    }else {
                        // ("-- Übergehe Filter ".$key." bei Stufe ".$maxStufe);
                    }
            }            
        } // Where-Klausel
        
        
        // Historisierung der Datenquellen
        if (false) $tQuery = new Query('L');
        if (false) $tJoin = new JoinedTable('');
        if (! $baseTable && $tQuery->histType == Query::VONBIS 
            && isValid($tJoin->dimHistoryQueryDate[0])){
            $mr = $tQuery->getReportMetaRecord($tQuery->histAb);
            $sql->addWhere($mr->selectVar." <= ".$tJoin->dimHistoryQueryDate[0]);
            $mr = $tQuery->getReportMetaRecord($tQuery->histBis);
            $sql->addWhere($mr->selectVar." >= ".$tJoin->dimHistoryQueryDate[0]);
        }else if ($baseTable && $query->histType == Query::VONBIS
            && isValid($query->historyQueryDate)){
            $mr = $query->getReportMetaRecord($query->histAb);
            $sql->addWhere($mr->selectVar." <= ".$query->historyQueryDate);
            $mr = $query->getReportMetaRecord($query->histBis);
            $sql->addWhere($mr->selectVar." >= ".$query->historyQueryDate);            
        }else if ($tQuery->histType == Query::STICHTAGE 
            && isValid($tQuery->histStichtag)){
            $mr = $tQuery->getReportMetaRecord($tQuery->histStichtag);
            if ($mr != null){
                if (false) $tJoin = new JoinedTable('');
                $histOption = $query->historyOption;
                if (! $baseTable) $histOption = $tJoin->dimHistoryOption[0];
                switch ($histOption){
                    case HistoryOptions::HSUEBERNEHMEN:
                            switch ($query->historyOption){
                                case HistoryOptions::HSSTICHTAGBASE:
                                case HistoryOptions::HSAKTUELL:
                                case HistoryOptions::HSDATUM:
                                case HistoryOptions::HSVJENDE:
                                case HistoryOptions::HSVVJENDE:
                                default:
                                    if ($baseTable) $sql->addWhere($mr->selectVar." = ".$query->historyQueryDate);
                                    else {
                                        $sql->addWhere($mr->selectVar." = ".$tJoin->dimHistoryQueryDate[0]);                                    
                                    }
                                    break;
                            }                        
                        break;
                    case HistoryOptions::HSSTICHTAGBASE:    
                        if ($joinWerte) $sql->addWhere($mr->selectVar." = ".$tJoin->dimHistoryQueryDate[0]);                       
                        break;
                    case HistoryOptions::HSAKTUELL:
                    case HistoryOptions::HSDATUM:
                    case HistoryOptions::HSVJENDE:
                    case HistoryOptions::HSVVJENDE:
                    default:
                        if ($baseTable) $sql->addWhere($mr->selectVar." = ".$query->historyQueryDate);
                        else $sql->addWhere($mr->selectVar." = ".$tJoin->dimHistoryQueryDate[0]);
                        break;
                }
            }
        }
        
        if ($testMode) $sql->addWhere('false');
        if ($colGroup >= 0 ) {
            $rc = $query->repColumns->getPos($colGroup);
            if ($rc==null){
                addMeldung($sql,"FEHLER keine ColGroup an Position $colGroup");
            }else
                if (isValid($rc->filter)) {   // && ! isValueFilter(rc.colFilter)) {
                    if (($baseTable && substr($rc->filter,1,1) != ':')
                        || (!$baseTable && substr($rc->filter,0,2) == $tJoin->kennzeichen.':')
                        ){
                            $sql->sql.="\n-- Column-Filter $colGroup : ".$rc->filter;
                            
                            $sql->addFilterSQL($rc->filter,$rc,$tJoin);
                    }else
                    if ($query->values_in_columns
                        && ! $baseTable
                        && substr($rc->filter,1,1) != ':'
                        && array_key_exists($rc->filter, $tQuery->metaData)
                        && $tQuery->getReportMetaRecord($rc->filter)->filterType != MetaRecord::NONE
                        ){
                            $sql->sql.="\n-- Column-Filter $colGroup : ".$rc->filter;
                            
                            $sql->addFilterSQL($tJoin->kennzeichen.':'.$rc->filter,$rc,$tJoin);
                    }
                    else  $sql->sql.="\n-- Kein Col-Filter an Position $colGroup\n";
                    
            }
        }
        
        // DA FEHLT NOCH WAS
        if ($maxStufe > 0) {
            $fg = true;
           
            for ($i=0; $i < $maxStufe; $i++) {
                $var = $query->repRows->getPos($i);
                if (!$var->webLinkColumn){
                    if ( ! gesperrteRow($var->getName(), $table)){
                        if ($fg)  $sql->sql.="\n group by S".$i;
                        $fg = false;
                        $mr = $query->getReportMetaRecord($var->getName());
                        if ($mr == null) {
                            error_log('FEHLER Metadaten nicht gefunden '.$var->getName());
                            continue;
                        }
                        $sql->sql.=",\n ";
                        $sql->sql.=$mr->getSelName();
                    } else  $sql->sql.="\n-- gesperrte Row ".$var->getName()." ".$sperrGrund."\n";
                }
            }
            if ($colGroup >= 0) {
                $rc = $query->repColumns->getPos($colGroup);
                if ($rc==null){
                    addMeldung($sql,"FEHLER2 keine ColGroup an Position $colGroup");
                }else  if ( ! gesperrteRow($rc->getVarName(), $table)){                 
                    if ($fg)  $sql->sql.="\n group by S".($maxStufe -1);
                    $fg = false;
                    $mrr = $query->getReportMetaRecord($rc->getName());
                    if (substr($rc->getName(),1,1) == ':') $sql->sql.=",\n  ".substr($rc->getName(),0,1).'__'.$mrr->getSelName();
                    else $sql->sql.=",\n  ".$mrr->getSelName();
                } else  $sql->sql.="\n-- gesperrte Row ".$rc->getName()." ".$sperrGrund."\n";
            }
        }else {
            if ($colGroup >= 0) {
                $rc = $query->repColumns->getPos($colGroup);
                if ($rc==null){
                    addMeldung($sql,"FEHLER3 keine ColGroup an Position $colGroup");
                }else if ( ! gesperrteRow($rc->getVarName(), $table)){
                    $mrr = $query->getReportMetaRecord($rc->getName());
                    $sql->sql.="\n group by S0";
                    if (substr($rc->getName(),1,1) == ':') $sql->sql.=",\n  ".substr($rc->getName(),0,1).'__'.$mrr->getSelName();
                    else $sql->sql.=",\n  ".$mrr->getSelName();
                } else  $sql->sql.="\n-- gesperrte Row ".$rc->getName()." ".$sperrGrund."\n";
            }
        }
        
        if (array_key_exists('test',$_REQUEST)){
            echo '<pre>SelectVars: ';
            var_dump($selectVars);
            echo "</pre><br>\n";
        }
        return $sql;
}

/**
 * 
 * @param string $name
 * @param SqlAbfrage $sql
 * @return boolean
 */
function createTable($name, $sql){
    global $dbDWH, $testMode;
//    require_once './phpsec/createDBH.inc';
    if ($dbDWH == null) setdbDWH();
    $dbDWH->exec('drop temporary table if exists WSR_'.$name);
    $sql->sql = 'create temporary table WSR_'.$name." as \n".trim($sql->sql);
    $stmt = $sql->execute();
    if ($stmt === false){
        if ($testMode){
            echo "Fehler beim Erstellen der Tabelle $name:\n";
            echo $sql->errorInfo()[2]."\n";
        }else{
            echo "<br><b>Fehler beim Erstellen der Tabelle $name:</b></br><pre>\n";
            echo $sql->errorInfo()[2]."<br>";
            var_dump($sql);
            echo "</pre>";            
        }
        return false;
    }
    return true;
}

function dumpTable($name){
    global $dbDWH;
    require_once './phpsec/createDBH.inc';
    if ($dbDWH == null) setdbDWH();
    $stmt = $dbDWH->query('select * from WSR_'.$name);
    if ($stmt === false){
        echo "<br><b>Fehler in der Abfrage:</b></br><pre>\n";
        echo $dbDWH->errorInfo()[2];
        echo "</pre>";
        return;
    }
    $daten = $stmt->fetchAll(PDO::FETCH_ASSOC);
    echo '<pre>'; var_dump($daten); echo '</pre>';
    $stmt->closeCursor();
}


function showTable($name){
    global $dbDWH;
    require_once './phpsec/createDBH.inc';
    if ($dbDWH == null) setdbDWH();    
    $stmt = $dbDWH->query('select * from WSR_'.$name);
    if ($stmt === false){
        echo "<br><b>Fehler in der Abfrage:</b></br><pre>\n";
        echo $dbDWH->errorInfo()[2];
        echo "</pre>";
        return;
    }
    $first = true;
    while ($z = $stmt->fetch(PDO::FETCH_NUM)){
        if ($first){
            echo "\n<table id=\"$name\"><thead><tr>";
            for ($i = 0; $i < count($z); $i++)
                echo '<th>'.$stmt->getColumnMeta($i)['name'].'<br>'.$stmt->getColumnMeta($i)['native_type'].'</th>';
                echo '</tr></thead>';
        }
        $first = false;
        echo "\n<tr>";
        for ($i = 0; $i < count($z); $i++){
            echo '<td>'.$z[$i].'</td>';
        }
        echo '</tr>';
    }
    echo '</table>';
    $stmt->closeCursor();
}

/**
 * Bei JOINOUTERWERTE muss zunächst über Union eine
 * führende Tabelle, lead, erstellt werden, die alle Zeilen 
 * aus beiden Tabellen genau einmal enthält.
 * 
 * Um Code-Varianten zu vermeiden, und weil es nicht viel
 * kostet, wird aus auch gemacht, wenn es keine zweite Tabelle gibt.
 * 
 * @param SqlAbfrage $sql
 */

function createLeadTable($sql){
    global $meldungen, $query;
    
    // Join mit Werten
    $sql->sql = "\n-- Lead Table für ".$query->title."\n";
    //   $jt = new JoinedTable(new AddedQuery(null));
    
    if (false) $mr = new MetaRecord();
    
    $sql->sql.="select distinct Stufe, colGroup";
    for ($i = 0; $i < $query->repRows->size(); $i++) {
        if (!$query->repRows->getPos($i)->webLinkColumn){
            $var = $query->repRows->getPos($i)->getName();
            $varSel = $query->repRows->getPos($i)->getSelName();
            $sql->sql.=",\nS".$i;
            if (array_key_exists($var, $query->metaData)){
                $sql->sql.=", ".$varSel;
            }else{
                $sql->sql.=", null as ".$varSel;
                addMeldung($sql,"FEHLER createLeadTable() repRow = "
                    .$var." hat keine Metadaten");
            }
        }
    }
    if ($query->repRows->size() == 0) $sql->sql.=", S0";
    //    Vector<String> addValues = new Vector<String>();
    $tempColumns = array();

    for ($i = 0; $i < $query->repColumns->size(); $i++) 
        if ($query->values_in_columns || substr($query->repColumns->getPos($i)->val_name, 1,1) == ':' ){
        $rc = $query->repColumns->getPos($i);
        if ($rc != null) {
            if (array_key_exists($rc->getVarName(), $tempColumns)) continue;
            $tempColumns[$rc->getVarName()] = true;
            $mr = $query->getReportMetaRecord($rc->column_name);
            if ($mr == null) {
                addMeldung($sql,"FEHLER createJoinedTable() rc->column_name = "
                    .$rc->column_name." hat keine Metadaten");
            }else{
                if (substr($rc->column_name,1,1)== ':') {
                    $sql->sql.=",\n ".substr($rc->column_name,0,1).'__'.$mr->getSelName();
                }
                else {
                    $sql->sql.=",\n ".$mr->getSelName();
                }
            }
        }
    }
    
    $sql->sql.="\n from WSR_base";
   
    // Jetzt kommen die Werte-Tabellen mit JOINOUTERWERTE
    for ($r = 0; $r < $query->repTables->size(); $r++){
        $rt = $query->repTables->getPos($r);
        if (false) $rt = new JoinedTable($query);
        if ($rt->joinType == JoinedTable::JOINOUTERWERTE){
            $jMeta = $rt->query->metaData;
            $addToLead = true;
// Zuerst prüfen, ob alle Variablen vorhanden sind.
// Z.B. ist bei gesperrten Zeilen hier der Union NICHT möglich.
            for ($i = 0; $i < $query->repRows->size(); $i++) {
                if (!$query->repRows->getPos($i)->webLinkColumn){
                    $var = $query->repRows->getPos($i)->getName();
                    if (!array_key_exists($var, $jMeta)) {
                        $sql->sql.="\n-- Metadaten fehlen für Row {$var} bei {$rt->kennzeichen}";
                        $addToLead = false;
                    }
                    if (gesperrteRow($var, $rt->kennzeichen)) {
                        $sql->sql.="\n-- gesperrte Row {$var} bei {$rt->kennzeichen}";
                        $addToLead = false;
                    }
                }
            }
            
                for ($i = 0; $i < $query->repColumns->size(); $i++) 
                    if ($query->values_in_columns || substr($query->repColumns->getPos($i)->val_name, 1,1) == ':' ){
                        $rc = $query->repColumns->getPos($i);
                if ($rc == null)  $addToLead = false;
                    $mr = $query->getReportMetaRecord($rc->column_name);
                    if ($mr == null)  {
                        $sql->sql.="\n-- Col-Metarecord fehlt {$rc->getName()} bei {$rt->kennzeichen}";
                        $addToLead = false;
                    }
                    if (gesperrteRow($rc->getName(), $rt->kennzeichen)) {
                        $sql->sql.="\n-- gesperrte Col {$rc->getName()} bei {$rt->kennzeichen}";
                        $addToLead = false;
                    }
            }
            if (! $addToLead) continue;
// Jetzt die SQL erzeugen
            $sql->sql.="\n union select Stufe, colGroup";
            for ($i = 0; $i < $query->repRows->size(); $i++) {
                if (!$query->repRows->getPos($i)->webLinkColumn){
                    $var = $query->repRows->getPos($i)->getName();
                    $varSel = $query->repRows->getPos($i)->getSelName();
                    $sql->sql.=",\nS".$i;
                    if (array_key_exists($var, $jMeta)){
                        $sql->sql.=", ".$varSel;
                    }else{
                        $sql->sql.=", null as ".$varSel;
                        addMeldung($sql,"FEHLER createLeadTable() {$rt->kennzeichen} repRow = "
                            .$var." hat keine Metadaten");
                    }                    
                }
            }
            if ($query->repRows->size() == 0) $sql->sql.=", S0";
            //    Vector<String> addValues = new Vector<String>();
            $tempColumns = array();
            if ($query->values_in_columns)
                for ($i = 0; $i < $query->repColumns->size(); $i++) {
                $rc = $query->repColumns->getPos($i);
                if ($rc != null) {
                    if (array_key_exists($rc->getVarName(), $tempColumns)) continue;
                    $tempColumns[$rc->getVarName()] = true;
                    $mr = $query->getReportMetaRecord($rc->column_name);
                    if ($mr == null) {
                        addMeldung($sql,"FEHLER createJoinedTable() rc->column_name = "
                            .$rc->column_name." hat keine Metadaten");
                    }else{
                        if (substr($rc->column_name,1,1)== ':') {
                            $sql->sql.=",\n null as ".substr($rc->column_name,0,1).'__'.$mr->getSelName();
                        }
                        else if ($query->values_in_columns){
                            $sql->sql.=",\n ".$mr->getSelName();
                        }
                        else{
                            $sql->sql.=",\n null as ".$mr->getSelName();
                        }
                    }
                }
            }
            $sql->sql.="\n from WSR_".$rt->kennzeichen;            
        }
    }
} // ENDE createLeadTable



function createJoinedTable($sql){
    global $meldungen, $query;
    $variablen = array();

// Join mit Werten    
    $sql->sql = "\n-- Joined Table für ".$query->title."\n";
    //   $jt = new JoinedTable(new AddedQuery(null));
    
    if (false) $mr = new MetaRecord();
    if (false) $mro = new MetaRecord();
    
    $sql->sql.="select lead.Stufe, lead.colGroup";
    for ($i = 0; $i < $query->repRows->size(); $i++) {
        $var = $query->repRows->getPos($i);
        if (!$query->repRows->getPos($i)->webLinkColumn){
            $mr = $query->getReportMetaRecord($var->getName());
            $sql->sql.=",\nlead.S".$i;
            if (! array_key_exists($mr->getName(), $variablen)){
                $variablen[$mr->getName()] = 1;
                $sql->sql.=", lead.".$mr->getSelName();
                // TODO
                if ($mr->hasDispVar()) selectJoinedDispName($mr,$sql);
                
    //            if ($mr->hasDispVar()) $sql->sql.=", ifnull(base.".$mr->getDispName().',A.'.$mr->getDispName().') as '.$mr->getDispName();
            }
        }
        if (isValid($var->web_link)
            && substr($var->web_link,0,1) == '#'){
            $mro = $query->getReportMetaRecord(trim(substr($var->web_link,1)));
            if ($mro != null) {
                if (! array_key_exists($mro->getName(),$variablen)) {
                    $variablen[$mro->getName()] = 1;
                    selectJoinedName($mro, $sql);
                    if (isValid($mro->displayVar)) selectJoinedDispName($mro, $sql);
                }
            }
        }
    }
    if ($query->repRows->size() == 0) $sql->sql.=", lead.S0";
    
    /*
     * Hier ggf. Wert-Anzeige für Sonderfilter selektieren
     */
    for ($i=0; $i < $query->repFilter->size(); $i++){
        $rfilter =  $query->repFilter->getPos($i);
        if (false) $rfilter = new RepFilter('');
        if (! $rfilter->isSet()) continue;
        if ($rfilter->isValueFilter()) continue;
        if ($rfilter->isWindowFilter()) continue;
        $vn = $rfilter->getName();
        $mr = $query->getReportMetaRecord($vn);
        if ($mr == null){
            error_log('FEHLER: Metarecord für '.$rfilter->getName().' nicht gefunden');
        }
        if (false) $mr = new MetaRecord();
        if (($mr->filterType == MetaRecord::SONDERFILTER
            || $mr->filterType == MetaRecord::SONDERFILTERSEL)
            && isValid($mr->displayVar)){
                if (substr($vn,1,1) == ':'){ // Wert aus/für Join-Werte-Tabelle
                    $sql->sql .= ",\n ".substr($vn,0,1).'.FD_'.str_replace(':', '_', $vn);
                }else {
                    $sql->sql .= ",\n base.FD_".$vn;
            }
        }
    }
    
    for ($i = 0; $i < $query->repValues->size(); $i++) {
        /**
         * @var RepValue $var
         */
        $var = $query->repValues->getPos($i);
        $vn = $var->getValueName();
        $mr = $query->getReportMetaRecord($vn);
        if ($mr == null){
            addMeldung($sql,"FEHLER keine Metadaten für $vn bei Report-Values");
            continue;
        }
        
            switch (substr($vn,1,1)) {
                case '=':
                    // ZUERST die Basis-Werte zum ggf. Debuggen holen
                    $vl = getCalcBaseVars($mr, 'base');
                    foreach ($vl as $vln){
//                        $vln = substr($vln,5);
//????
                        if ($var->base == 'base') $mro = $query->getReportMetaRecord($vln);
                        else $mro = $query->getReportMetaRecord($var->base.':'.$vln);
                        if ($mro == null){
                            addMeldung($sql,"FEHLER keine Metadaten für $vln bei $vn");                                                        
                        }else{
                            if (($var->base != 'base') &&  ! array_key_exists($var->base.'__'.$mro->getName(), $variablen)){
                                $variablen[$var->base.'__'.$mro->getName()] = 1;
                                $sql->sql.=",\n".$var->base.'.'.$mro->getName();
                                $sql->sql.=' as '.$var->base.'__'.$mro->getName();                                
                            }else if (($var->base == 'base') &&  ! array_key_exists($mro->getName(), $variablen)) {
                                $variablen[$mro->getName()] = 1;
                                $sql->sql.=",\n".$var->base.'.'.$mro->getName();
                            }
                        }
                    }
                    $k = substr($vn,0,1);
                    $vl = getCalcBaseVars($mr, 'join');
                    foreach ($vl as $vln){
//                        $vln = substr($vln,5);
                        $mro = $query->getReportMetaRecord($k.':'.$vln);
                        if ($mro == null){
                            addMeldung($sql,"FEHLER keine Metadaten für $k.__$vln bei $vn");
                        }else{
                            if (! array_key_exists($k.'__'.$mro->getName(), $variablen)) {
                                $variablen[$k.'__'.$mro->getName()] = 1;
                                $sql->sql.=",\n".$k.'.'.$mro->getName().' as '.$k.'__'.$mro->getName();
                            }
                        }
                    }
                    // Jetzt berechnen
                    $f = str_replace('join.',$k.'.', $mr->selectVar);
                    if ($var->base != 'base') $f = str_replace('base.',$var->base.'.', $f);
                    if ($var->isWindowValue()) $ergName = $var->getName();
                    else $ergName = $k.'__'.$mr->getName();
                    if (! array_key_exists($ergName, $variablen)) {
                        $variablen[$ergName] = 1;
                        $sql->sql .= ",\n".$f.' as '.$ergName;
                    }
                    break;
                case ':':
                    $k = substr($vn,0,1);
//                    if ($var->isWindowValue()) $ergName = str_replace('=','__',str_replace(':','__',$var->getValueName()));
   //                 else 
                        $ergName = $k.'__'.$mr->getName();
                    if (! array_key_exists($ergName, $variablen)) {
                        $variablen[$ergName] = 1;
                        $sql->sql.=",\n".str_replace(':','.',$var->getValueName()).' as '.$ergName;
                    }                    
                    break;
                    
                default:
                    if (! array_key_exists($var->getValueName(), $variablen)) {
                        $variablen[$var->getValueName()] = 1;
                        $sql->sql.=",\n base.".$var->getValueName().' as '.$var->getValueName();
                            
/*                            
                            $sql->sql.=",\n".$var->function."(base.".$mr->getSelName().') over (partition by lead.Stufe';
                            
                            // , lead.S0 ';
                            // $sql->sql .= 'order by lead.S_Gebiet, lead.S1, lead.S_Region ';
                            for ($ic=0; $ic < $query->repRows->size(); $ic++){
                                $rRow = $query->repRows->getPos($ic);
                                if ( $rRow->webLinkColumn) continue;                                    
                                $sql->sql .= ', lead.S'.$ic;
                                if ($ic == $var->abSpalte) $sql->sql .= ' order by ';
                                else $sql->sql .= ', ';
                                $sql->sql .= 'lead.'.$query->getReportMetaRecord($rRow->getName())->getSelName();
                            }
                            
                            $sql->sql .= ')';
//                            $sql->sql .= "\n-- abSpalte = ".$var->abSpalte."\n";
                        }
                        else $sql->sql.=",\nbase.".$mr->getSelName();
                        if ($var->isWindowValue()) $sql->sql.=' as '.$var->getName();
                        */
                    }
                    ;
                    break;
        }
    }
    
    //    Vector<String> addValues = new Vector<String>();
    for ($i = 0; $i < $query->repColumns->size(); $i++) {
        $rc = $query->repColumns->getPos($i);
        if ($rc != null) {
            //				if (! addValues.contains(rc.columnName)) {
            //					addValues.add(rc.columnName);
            $mr = $query->getReportMetaRecord($rc->column_name);
            if ($mr == null) {
                addMeldung($sql,"FEHLER createJoinedTable() rc->column_name = "
                    .$rc->column_name." hat keine Metadaten");                
            }else {
                if (! array_key_exists($rc->getVarName(), $variablen)) {
                    $variablen[$rc->getVarName()] = 1;
                        if (substr($rc->column_name,1,1)== ':') {
                            if ($query->values_in_columns)
                                $sql->sql.=",\n lead.".substr($rc->column_name,0,1).'__'.$mr->getSelName();
                           else
                            $sql->sql.=",\n base.".substr($rc->column_name,0,1).'__'.$mr->getSelName();
                            // TODO für values_in_columns
                            if ($mr->hasDispVar()) $sql->sql.=", base.".substr($rc->column_name,0,1).'__'.$mr->getDispName();
                        }
                        else {
                            if ($query->values_in_columns){
                                $sql->sql.=",\n lead.".$mr->getSelName();
                                if ($mr->hasDispVar()) selectJoinedDispName($mr,$sql);
                            }
                            else {
                                $sql->sql.=",\n base.".$mr->getSelName();
                                if ($mr->hasDispVar()) $sql->sql.=",\n base.".$mr->getDispName();
                            }
                            // TODO für values_in_columns
                        }
                    }
                    if ( ! $query->values_in_columns
                        &&  isValid($rc->val_name) ) {
                            $mro = $query->getReportMetaRecord($rc->val_name);
                            if ($mro == null){
                                addMeldung($sql,"FEHLER createJoinedTable() rc->val_name = "
                                    .$rc->val_name." hat keine Metadaten");
                            }else {
                                $dispVar = $mro->getName();
                                $dispVarSel = $mro->getSelName();
                                if (substr($rc->val_name,1,1) == ':'
                                    && substr($rc->val_name,0,2) == substr($rc->getName(),0,2)) {
                                    $dispVar = substr($rc->val_name,0,1).'__'.$mro->getName();
                                    $dispVarSel = substr($rc->val_name,0,1).'__'.$mro->getSelName();
                                }
                                if (! array_key_exists($dispVar, $variablen)) {
                                    $variablen[$dispVar] = 1;                  
                                    if ((substr($rc->val_name,1,1) == ':' 
                                        || substr($rc->val_name,1,1) == '=')
                                     && substr($rc->getName(),1,1) != ':') 
                                        $sql->sql.=",\n".substr($rc->val_name,0,1).'.'.$dispVarSel
                                        .' as '.substr($rc->val_name,0,1).'__'.$dispVarSel;
                                else                                                
                                    $sql->sql.=",\n base.".$dispVarSel;
                                }
                            }
                        }
            }
        }
    }
    // WINDOWFILTER
    
    for ($i = 0; $i < $query->repFilter->size(); $i++){
        $repFilter = $query->repFilter->getPos($i);
        if ($repFilter->isWindowFilter() && $repFilter->isSet()){
            if (false) $repFilter = new WindowFilter('');
            $mr = $repFilter->getMetaRecord();
            $sql->sql .= ",\n".$mr->selectVar;
            $sql->sql .=' as WSR_WINDOWFILTER'.$i;
        }        
    }
    
    
    
    $sql->sql.=" from WSR_lead lead";
    /**
     * Joins über Row- und Col-Variablen
     * Hier werden schon die Namen aus den Metadaten verwendet. 
     * Deshalb funktioniert auch using.
     */
    $sql->sql .= "\n left join WSR_base base on"
    ." lead.Stufe = base.Stufe and lead.colGroup = base.colGroup";
    for ($i = 0; $i < $query->repRows->size(); $i++) {
        if (!$query->repRows->getPos($i)->webLinkColumn){            
            $var = $query->repRows->getPos($i)->getSelName();
            $sql->sql.="\n-- Join ".$var;
            $sql->sql.="\n and base.S".$i." = lead.S".$i;
            $sql->sql.="\n  and (base.$var = lead.{$var} or ";
            $sql->sql.=" (base.{$var} is null and  lead.{$var} is null)) ";
        }
    }

    $tempColumns = array();
   
        for ($i = 0; $i < $query->repColumns->size(); $i++) 
            if ($query->values_in_columns || substr($query->repColumns->getPos($i)->val_name,1,1) == ':'){
            $rc = $query->repColumns->getPos($i);
            if (array_key_exists($rc->getVarName(), $tempColumns)) continue;
            $tempColumns[$rc->getVarName()] = true;
            $vn = $rc->column_name;
            $mr = $query->getReportMetaRecord($vn);
            $vs = $mr->getSelName();
            if ($query->values_in_columns){
                
            }
            else if (substr($query->repColumns->getPos($i)->column_name,1,1) == ':'){
                    $vs = substr($query->repColumns->getPos($i)->column_name,0,1).'__'.$vs;
            }
            else
                if (substr($query->repColumns->getPos($i)->val_name,1,1) == ':'){
                    // $vs = substr($query->repColumns->getPos($i)->val_name,0,1).'__'.$vs;
            }
            if ($mr != null){
                $sql->sql.="\n-- Join ".$vn;
                $sql->sql.="\n  and (base.".$vs." = lead.".$vs.' or ';
                $sql->sql.=" (base.".$vs." is null and  lead.".$vs." is null)) ";
            }
             else addMeldung($sql,"Metadaten zur Spalte $vn fehlen");
        }
    
    for ($j = 0; $j < $query->repTables->size(); $j++) {
        $jt = $query->repTables->getPos($j);
        if ($jt->joinType == JoinedTable::JOINWERTE
            || $jt->joinType == JoinedTable::JOINOUTERWERTE) {
            $sql->sql.="\n  left join WSR_";  // ist in MariaDB immer outer join
            $sql->sql.= $jt->kennzeichen;
            $sql->sql.= ' '.$jt->kennzeichen;
            $sql->sql.=" on ".$jt->kennzeichen.".Stufe = lead.Stufe and ".$jt->kennzeichen.".colGroup = lead.colGroup";
            for ($i = 0; $i < $query->repRows->size(); $i++) {
                if (!$query->repRows->getPos($i)->webLinkColumn){
                    $var = $query->repRows->getPos($i);
                    $mr = $query->getReportMetaRecord($var->getName());
                    if ( ! gesperrteRow($var->getName(), $jt->kennzeichen)){
                        $sql->sql.="\n-- Join ".$var->getName();
                        $sql->sql.="\n and lead.S".$i." = ".$jt->kennzeichen.".S".$i;
                        $metaData = $jt->query->metaData;
                        if (array_key_exists($var->getName(), $metaData)){
                            $sql->sql.="\n  and (lead.".$mr->getSelName()." = ".$jt->kennzeichen.".".$mr->getSelName()." or ";
                            $sql->sql.=" (lead.".$mr->getSelName()." is null and  ".$jt->kennzeichen.".".$mr->getSelName()." is null)) ";
                        }
                        else addMeldung($sql,"Metadaten zur Zeile ".$var->getName()." fehlen");
                    }
                }
            }
            $tempColumns = array();
            
                for ($i = 0; $i < $query->repColumns->size(); $i++) 
                    if ($query->values_in_columns || substr($query->repColumns->getPos($i)->val_name,0,2) == $jt->kennzeichen.':'){
                    $rc = $query->repColumns->getPos($i);
                    if (array_key_exists($rc->getVarName(), $tempColumns)) continue;
                    $tempColumns[$rc->getVarName()] = true;                    
                    $vn = $rc->column_name;
                    if (! gesperrteRow($vn, $jt->kennzeichen)){
                        $sql->sql.="\n-- ".$vn.' in '.$jt->kennzeichen.' ist keine gesperrte Row';
                        $mr = $query->getReportMetaRecord($vn);
                        if ($mr != null){
                            $sql->sql.="\n-- Join ".$vn;
//                            $sql->sql.="\n and lead.S".$i." = ".$jt->kennzeichen.".S".$i;
                            $metaData = $jt->query->metaData;
                            if (array_key_exists($vn, $metaData)){
                                $sql->sql.="\n  and (lead.".$mr->getSelName()." = ".$jt->kennzeichen.".".$mr->getSelName().' or ';
                                $sql->sql.=" (lead.".$mr->getSelName()." is null and  ".$jt->kennzeichen.".".$mr->getSelName()." is null)) ";
                            }
                            else addMeldung($sql,"Metadaten zur Spalte $vn fehlen");
                            if (!$query->values_in_columns){
                                $sql->sql.="\n  and (lead.colGroup) = ".$jt->kennzeichen.'.colGroup ';
                            }
                        }
                    }
                }
            }
        }
        /*
         * Hier muss vorher in der Base-Tabelle die enstsprechende Colgroup angelegt sein
         * Aktuell fehlt sie noch
         * Alternative UNION, aber dann müssen die Value-Vars mit NULL deklariert werden.
         else if (jt.joinType == JOINSPALTEN) {
         $sql->sql.="\n  left join WSR_";
         $sql->sql.=$jt->kennzeichen);
         $sql->sql.=" on ".$jt->kennzeichen.".S0 = 99";
         }
         
         */
    
    
    // Werte-Filter
    
    $sql->sql .= "\nwhere lead.colGroup != 99 or (true ";
    $sql->firstWhere = false;
    for ($i=0; $i < $query->repFilter->size(); $i++) {
        $var = $query->repFilter->getPos($i);
        if (false) $var = new RepFilter('');
        $vn = $var->getName();
        if ($var->isWindowFilter()) continue;
        $mr = $query->getReportMetaRecord($vn);
        if ($mr == null) {
            addMeldung($sql,"FEHLER keine Metdadaten für Filter ".$i.": ".$vn);            
        }else 
        // Werte-Filter wirken erst bei selectSortedTable()
        // Werte aus Spalten (colGroup != 99) müssen ignoriert werden
        if (isValid($vn)
            && $var->isValueFilter()        
            ){  
            if ($var->abSpalte > 0){
                // Hier fehlt AB SPALTE
                $sql->sql .= "\n and (lead.Stufe < ".$var->abSpalte.' or (true ';
                if ($var->isJoinedFilter()) $sql->addFilterSql($vn,$var,substr($var->getName(),0,1));
                else $sql->addFilterSql($vn,$var,null);                
                $sql->sql .= '))';
            }else{
                    if ($var->isJoinedFilter()) $sql->addFilterSql($vn,$var,substr($var->getName(),0,1));
                    else $sql->addFilterSql($vn,$var,null);
            }
        }
    }
    $sql->sql .=')';
    

     
     //    System.out.println("\n".sql.toString());
     //    stmt.execute(sql.toString());
     //    System.out.println(";\n select * from WSR_".erg.";\n";
     
     return $sql;
}

function createMysqlSorterTable($sql, $baseTable, $nr) {
    global $query, $meldungen;
    $colGroup = "99";
    if ($query->sort_mode == SortOptions::NOSORTVAL) return null;
    // ?? $sortVar = sortValue;
    if (array_key_exists('sortDir', $_REQUEST)) $sortDirection = $_REQUEST['sortDir'];
    else $sortDirection = Sort::NONE;
    if ($sortDirection == Sort::NONE) return null;
    if (array_key_exists('sortVal', $_REQUEST)) $sortValue = $_REQUEST['sortVal'];
    else $sortValue = null;
    if (array_key_exists('sortCol', $_REQUEST)) $sortColGroup = $_REQUEST['sortCol'];
    else $sortColGroup = null;
    if (array_key_exists('sortColVal', $_REQUEST)) $sortColValue = $_REQUEST['sortColVal'];
    else $sortColValue = null;
    
    $query->sort_direction = $sortDirection;
    $query->sort_value = $sortValue;
    $query->sort_colGroup = $sortColGroup;
    $query->sort_colValue = $sortColValue;
    
    $sortVar = $sortValue;
    
    if (isValid($sortColGroup)) {
        $colGroup = $sortColGroup;
        $sortVar = $query->repColumns->getPos($sortColGroup)->val_name;
        $sortColVar = $query->repColumns->getPos($sortColGroup)->column_name;
    }
    
            if ($query->sort_mode == SortOptions::SORTLASTGROUP || $query->sort_mode == SortOptions::SORTTABLE) {
                $sql->sql = "select ";
                
                if (substr($sortVar,1,1) ==':' || substr($sortVar,1,1) =='=')
                    $sql->sql.= substr($sortVar,0,1).'__';
                    $sql->sql .= $query->getReportMetaRecord($sortVar)->getSelName();
                    $sql->sql .= " as sortValue, stufe ";
                    for ($i = 0; $i < $query->repRows->size(); $i++) {
                        if (!$query->repRows->getPos($i)->webLinkColumn){
                            $var = $query->repRows->getPos($i)->getName();
                            if (isValid($var)) {
                                $mr = $query->getReportMetaRecord($var);
                                $sql->sql .= ", ". $mr->getSelName();
                            }
                        }
                    }
                    $sql->sql .= "\n from WSR_";
                    $sql->sql .= $baseTable;
                    if ( $query->repColumns->size() > 0) {
                        $sql->addSel('colgroup = ', $colGroup);
                        if ($colGroup != 99  && isValid($sortColGroup)) {
/*                          $fv = new RepFilter($sortColGroup);
                            $fv->fs_von = $sortColValue;
                            $fv->fs_bis = $sortColValue;
  */                        // Hier funktioniert $sql->addFilterSql nicht !!
                            $mrc = $query->getReportMetaRecord($sortColVar);
                            $vn = $mrc->getName();
                            if (substr($sortColVar,1,1)==':') $vn = substr($sortColVar,0,1).'__'.$vn;
                            $sql->addSel($vn.' = ',$sortColValue);
                        }
                    }
                    if ($query->sort_mode == SortOptions::SORTLASTGROUP) {
                        $sql->addSel('stufe = ', $query->repRows->size());
                    }
                    return $sql;
            }
            
            if ($query->sort_mode == SortOptions::SORTSPECIAL || $query->sort_mode == SortOptions::SORTSPECIALSZ) {
                $s = $nr;
                    $sql->sql = "select ";
                    
                    if ( $query->sort_mode == SortOptions::SORTSPECIALSZ
                        || $s == $query->repRows->size() -1
                        || $query->repRows->getPos($s+1)->summenZeilen ){
                            // baseTable verwenden
                            if (substr($sortVar,1,1) ==':' || substr($sortVar,1,1) =='=')
                                $sql->sql.= substr($sortVar,0,1).'__';
                                $sql->sql .= $query->getReportMetaRecord($sortVar)->getSelName();
                                $sql->sql .= " as sortValue";
                                for ($i = 0; $i <= $s; $i++) {
                                    if (!$query->repRows->getPos($i)->webLinkColumn){                                
                                        $mr = $query->getReportMetaRecord($query->repRows->getPos($i)->getName());
                                        $sql->sql .= ", ";
                                        $sql->sql .= $mr->getSelName();
                                    }                                    
                                }
                                $sql->sql .= "\n from WSR_";
                                $sql->sql .= $baseTable;
                                $sql->addSel("Stufe = ", ($s+1)) ;
                                if ( $query->repColumns->size() > 0) $sql->addSel("colgroup = ", $colGroup);
                                if ($s < $query->repRows->size() -1) $sql->addWhere("S".($s+1));
                                
                                if ($colGroup != 99 && isValid($sortColGroup)) {
                                    $mrc = $query->getReportMetaRecord($sortColVar);
                                    $vn = $mrc->getSelName();                                    
                                    if (substr($sortColVar,1,1)==':') $vn = substr($sortColVar,0,1).'__'.$mrc->getSelName();
                                    $sql->addSel($vn.' = ',$sortColValue);
                                }
                    }else {
                        // Summen-Werte neu ermitteln
                        if (substr($sortVar,1,1) ==':' || substr($sortVar,1,1) =='=')
                            $sql->sql.= substr($sortVar,0,1).'__';
                            $sql->sql .= $query->getReportMetaRecord($sortVar)->getSelName();
                            $sql->sql .= " as sortValue";
                            for ($i = 0; $i <= $s; $i++) {
                                if (!$query->repRows->getPos($i)->webLinkColumn){
                                    $sql->sql .= ", ";
                                    $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                                }
                            }
                            $sql->sql .= "\n from WSR_";
                            $sql->sql .= $baseTable;
                            if (isValid($sortColGroup) && isValid($sortColValue)) {
                                $mrc = $query->getReportMetaRecord($sortColVar);
                                $vn = $mrc->getSelName();                                
                                if (substr($sortColVar,1,1)==':') $vn = substr($sortColVar,0,1).'__'.$vn;
                                $sql->addSel($vn.' = ',$sortColValue);
                                $rc = $query->repColumns->getPos($sortColGroup);
                                if ($rc->filter != null) {
                                    $sql ->addFilterSQL($rc->filter,$rc,null);
                                }
                            }
                            $sql->sql .= "\ngroup by ";
                            $mrr = $query->getReportMetaRecord($query->repRows->getPos(0)->getName());
                            $sql->sql .= $mrr->getSelName();
                           
                            
                            for ($i = 1; $i <= $s; $i++) {
                                if (!$query->repRows->getPos($i)->webLinkColumn){                                        
                                    $sql->sql .= ", ";
                                    $mrr = $query->getReportMetaRecord($query->repRows->getPos($i)->getName());
                                    $sql->sql .= $mrr->getSelName();
                                }
                                
                            }
                            
                    }
                    
                    return $sql;
                }
                addMeldung($sql,"Unbekannter query->sort_mode = ".$query->sort_mode);
                return null;
            
            
        
        
}
// ENDE createMysqlSorter ----------------------------------------------

function createSortedTable($sql,$baseTable){
    global $query, $meldungen, $mysql;
    
    $sortNachWert = false;
    $colGroup = "99";
    // ?? $sortVar = sortValue;
    if (array_key_exists('sortDir', $_REQUEST)) $sortDirection = $_REQUEST['sortDir'];
    else $sortDirection = 0;
    if (array_key_exists('sortVal', $_REQUEST)) $sortValue = $_REQUEST['sortVal'];
    else $sortValue = null;
    if (array_key_exists('sortCol', $_REQUEST)) $sortColGroup = $_REQUEST['sortCol'];
    else $sortColGroup = null;
    if (array_key_exists('sortColVal', $_REQUEST)) $sortColValue = $_REQUEST['sortColVal'];
    else $sortColValue = null;
    
    $query->sort_direction = $sortDirection;
    $query->sort_value = $sortValue;
    $query->sort_colGroup = $sortColGroup;
    $query->sort_colValue = $sortColValue;
    
    $sortVar = $sortValue;
    
    if (isValid($sortColGroup)) {
        $colGroup = $sortColGroup;
        $sortVar = $query->repColumns->getPos($sortColGroup)->val_name;
        $sortColVar = $query->repColumns->getPos($sortColGroup)->column_name;
    }
    
    $sortOrderBy = '';
    
    
    if ($sortDirection == Sort::NONE || $query->sort_mode == SortOptions::NOSORTVAL) {
        $sortOrderBy .= "\norder by ";
        for($i = 0; $i < $query->repRows->size(); $i++) {
            if (!$query->repRows->getPos($i)->webLinkColumn){
                $var = $query->repRows->getPos($i);
                if (!$var->webLinkColumn){
                    $sortOrderBy .= "\n base.S".$i.', ';
                    $mr = $query->getReportMetaRecord($var->getName());
                    switch ($var->sort){
                        case Sort::SORTANZEIGE     : $sortOrderBy .= 'base.'.$mr->getDispName();
                        break;
                        case Sort::SORTANZEIGEDESC : $sortOrderBy .= 'base.'.$mr->getDispName()." desc";
                        break;
                        case Sort::SORTINTERNDESC  : $sortOrderBy .= 'base.'.$mr->getSelName().' desc';
                        break;
                        default: $sortOrderBy .= 'base.'.$mr->getSelName();
                    }
                    $sortOrderBy .= ', ';
                }
            }
        }
        $sortOrderBy .= "\ncolGroup";
    }else{
        if (isValid($sortColGroup) && isValid($sortColValue)  ) {
        //sortierung nach Spalten-Wert
         }else
        if ($sortDirection != Sort::NONE && isValid($sortValue)) {
            //sortierung nach Wert
        }else {
            
        }
        
        $imax = $query->repRows->size();
        while ($imax > 1 && $query->repRows->getPos($imax-1)->webLinkColumn){
            $imax --;
        }
        $sortOrderBy .= "\norder by S0";
        for($i = 0; $i < $imax; $i++) {
            if (!$query->repRows->getPos($i)->webLinkColumn){
                $sortOrderBy .= ",\n";
//                if ($sortNachWert) {
                    if ($i == 0 && $query->sort_mode != SortOptions::SORTLASTGROUP) {
                        if ($sortDirection == Sort::AB)
                            $sortOrderBy .= "sort0.sortValue desc, ";
                            else
                                $sortOrderBy .= "sort0.sortValue, ";
                    }else if ($query->sort_mode == SortOptions::SORTSPECIAL
                        || $query->sort_mode == SortOptions::SORTSPECIALSZ) {
                            if ($sortDirection == Sort::AB)
                                $sortOrderBy .= "sort".$i.".sortValue desc, ";
                                else
                                    $sortOrderBy .= "sort".$i.".sortValue, ";
                        }else if ($i == $imax -1 && $query->sort_mode == SortOptions::SORTLASTGROUP) {
                            if ($sortDirection == Sort::AB)
                                $sortOrderBy .= "sort0.sortValue desc, ";
                                else
                                    $sortOrderBy .= "sort0.sortValue, ";
                        }
//                }
                $var = $query->repRows->getPos($i);
                $mr = $query->getReportMetaRecord($var->getName());
                $sortOrderBy .= "\n-- Sort ".$var->getName().' '.$var->sort."\n";
                switch ($var->sort){
                    case Sort::SORTANZEIGE     : $sortOrderBy .= $mr->getDispName();
                    break;
                    case Sort::SORTANZEIGEDESC : $sortOrderBy .= $mr->getDispName()." desc";
                    break;
                    case Sort::SORTINTERNDESC  : $sortOrderBy .= $mr->getSelName().' desc';
                    break;
                    default: $sortOrderBy .= $mr->getSelName();
                }
                
                while ($i < $query->repRows->size()-1 && $query->repRows->getPos($i+1)->webLinkColumn){
                    $i++;
                }
                if ($i < $query->repRows->size()-1) {
                    $sortOrderBy .= ", S".($i+1);
                }
            }
        }
        //		if ( qColumns.size() > 0)
        $sortOrderBy .= ", colGroup";
}
    
    
    $sql->sql = 'select ';
    if (($query->sort_mode == SortOptions::SORTSPECIAL || $query->sort_mode == SortOptions::SORTSPECIALSZ)
        && $sortDirection != Sort::NONE
        //    && isValid($sortValue)
        ) {
            
            // Zum Debuggen
            for ($s=0; $s < $query->repRows->size(); $s++) {
                if (!$query->repRows->getPos($s)->webLinkColumn){
                    $sql->sql .= "sort".$s.".sortValue as sort_".$s."_sortValue, ";
                }
            }
        }
        $sql->sql .= " base.* ";
        for ($iv = 0; $iv < $query->repValues->size(); $iv ++){
            /**
             * @var RepValue $rv
             */
            $rv = $query->repValues->getPos($iv);
            
            if ($rv->isWindowValue()) {
                $so = $sortOrderBy;
                if ($query->values_in_columns){
                    for ($ic=0; $ic < $query->repColumns->size();$ic++){
                        $rc = $query->repColumns->getPos($ic);
                        $mrv = $query->getReportMetaRecord($rc->getName());
                        switch ($rc->sort){
                            case Sort::SORTINTERN: $so .=', '.$mrv->getSelName();
                            break;
                            case Sort::SORTINTERNDESC: $so .=', '.$mrv->getSelName().' desc';
                            break;
                            case Sort::SORTANZEIGE: $so .=', '.$mrv->getDispName();
                            break;
                            case Sort::SORTANZEIGEDESC: $so .=', '.$mrv->getDispName().' desc';
                            break;
                            default: $so .=', '.$mrv->getSelName();
                            break;
                        }
                    }
                }
                $sql->sql .= "\n,".$rv->getFunctionSelect($so).' as '.$rv->getName();
            }
        }
        
        for ($iv = 0; $iv < $query->repColumns->size(); $iv ++){
            /**
             * @var RepColumn $rc
             */            
            $rc = $query->repColumns->getPos($iv);
            if ($rc->function <= 0) continue;
            if (!isValid($rc->val_name)) continue;
            /**
             * @var RepValue $rv
             */
            $rv = new RepValue($rc->val_name);
            $rv->function = $rc->function;
            $rv->value = $rc->val_name;
            $rv->abSpalte = $rc->abSpalte;
            $rv->nachSpalten = $rc->nachSpalten;
            $so = $sortOrderBy;
/*
            $mrv = $query->getReportMetaRecord($rc->getName());
            $vn = $mrv->getSelName();
            $vnd = $mrv->getDispName();
            if (substr($rc->getName(),1,1) == ':'){
                error_log('Check Join für '.$rc->getName());
                if ($query->repTables->get(substr($rc->getName(),0,1))->joinType == JoinedTable::JOINSPALTEN){
                $vn = substr($rc->getName(),0,1).'__'.$vn;
                $vnd = substr($rc->getName(),0,1).'__'.$vnd;
            }           
            error_log('Spalte '.$rc->getName().' VN='.$rc->getVarName().' SORT='.$rc->sort
                .' JT='.$query->repTables->get(substr($rc->getName(),0,1))->joinType == JoinedTable::JOINSPALTEN
                .' VNS='.$vn);
            }else{
                error_log('Base-Spalte '.$rc->getName().' VN='.$rc->getVarName().' SORT='.$rc->sort
                    .' VNS='.$vn);                
            }
/*            
            switch ($rc->sort){
                case Sort::SORTINTERN: $so .=', '.$vn;
                break;
                case Sort::SORTINTERNDESC: $so .=', '.$vn.' desc';
                break;
                case Sort::SORTANZEIGE: $so .=', '.$vnd;
                break;
                case Sort::SORTANZEIGEDESC: $so .=', '.$vnd.' desc';
                break;
                default: $so .=', '.$mrv->getSelName();
                break;
            }
            error_log('SO='.$so);
            error_log('SortOrder='.$sortOrderBy);
            $so = $sortOrderBy;
            */
            $sql->sql .= "\n,".$rv->getFunctionSelect($so).' as WSR_WINDOWCOLVALUE'.$iv;
        }
        
        
        $sql->sql .= "\n from WSR_";
        $sql->sql .= $baseTable;
        $sql->sql .= " base ";
        
        if ($query->hasReportShowFilter()){
            // Filter über 99er-Sätze
            $sql->sql .= "\n join WSR_".$baseTable.' s on s.colGroup = 99 and s.Stufe = base.Stufe ';
            for ($i=0; $i < $query->repRows->size(); $i++){
                $row = $query->repRows->getPos($i);
                if (false) $row = new RepRow('');
                if ($row->webLinkColumn) continue;
                $sql->sql .= "\n and base.S".$i.' = s.S'.$i.' and ( base.S_'.$row->getName().' = s.S_'.$row->getName()
                .' or ( base.S_'.$row->getName().' is null and s.S_'.$row->getName().' is null))';
            }
            $sql->firstWhere = false;
            for ($i = 0; $i < $query->repFilter->size(); $i++){
                $repFilter = $query->repFilter->getPos($i);
                if (false) $repFilter = new WindowFilter('');
                if ($repFilter->isWindowFilter() && $repFilter->isSet()){
                    if ($repFilter->getType() == WindowFilter::NTILE){
                        $t = explode('/',$repFilter->fs_von);
                        $v = (float) $t[0];
                        $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' <=',$v);
                    }else{
                    $fw = trim ($repFilter->fs_von);
                    switch (substr($fw, 0,2)){
                        case '<=' : $vt = str_replace(',','.', str_replace('.','',substr($fw, 2)));
                        $v =  (float) $vt;
                        if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                        if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                        $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' <=',$v);
                        break;
                        case '>=' : $vt = str_replace(',','.', str_replace('.','',substr($fw, 2)));
                        $v =  (float) $vt;
                        if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                        if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                        $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' >=',$v);
                        break;
                        case '==' : $vt = str_replace(',','.', str_replace('.','',substr($fw, 2)));
                        $v =  (float) $vt;
                        if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                        if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                        $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' =',$v);
                        break;
                        case '!=' : $vt = str_replace(',','.', str_replace('.','',substr($fw, 2)));
                        $v =  (float) $vt;
                        if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                        if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                        $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' !=',$v);
                        break;
                        default:
                            switch (substr($fw, 0,1)){
                                case '<' :  $vt = str_replace(',','.', str_replace('.','',substr($fw, 1)));
                                $v =  (float) $vt;
                                if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                                if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                                $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' <',$v);
                                break;
                                case '>' :  $vt = str_replace(',','.', str_replace('.','',substr($fw, 1)));
                                $v =  (float) $vt;
                                if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                                if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                                $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' >',$v);
                                break;
                                case '=' :  $vt = str_replace(',','.', str_replace('.','',substr($fw, 1)));
                                $v =  (float) $vt;
                                if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                                if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                                $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' =',$v);
                                break;
                                default: $vt = str_replace(',','.', str_replace('.','',$fw));
//                                error_log("RF-Typ = ".$repFilter->getType());
                                $v =  (float) $vt;
                                if ($repFilter->getType() == WindowFilter::PCTRANG) $v /= 100.0;
                                if ($repFilter->getType() == WindowFilter::ASUM) $v /= 100.0;
                                if ($repFilter->isSortFilter()) $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' <=',$v);
                                else $sql->addSel('(s.WSR_WINDOWFILTER'.$i.' >=',$v);
                                break;
                            }
                    }
                    }
                    if ($repFilter->isSortFilter()) {
                        $sEnde = $query->repRows->size()-1;
                        while ($sEnde > 0 && $query->repRows->getPos($sEnde)->webLinkColumn) $sEnde --;
                        $sql->sql .= ' or s.S'.($sEnde).')';
                    }
                    else $sql->sql .= ' or s.S'.($repFilter->abSpalte -1).')';
                }
            }
        }
        
    // Keine Sortierung nach Werten    
    if ($sortDirection == Sort::NONE || $query->sort_mode == SortOptions::NOSORTVAL) {
        $sql->sql .= $sortOrderBy;
        return $sql;
    }
    
    // Sortieren nach Werten    
        if ($sortDirection != Sort::NONE) {
            if ($query->sort_mode == SortOptions::SORTLASTGROUP || $query->sort_mode == SortOptions::SORTTABLE) {
                if (isset($mysql)){
                    $sql->sql .= "\nleft join WSR_sorter sort0 \non ";                    
                }else{
//                    error_log('SORT LAST GROUP');
                    $sql->sql .= "\n-- SORT LAST GROUP ";
                    
                    $sql->sql .= "\nleft join (select ";
                    $sql->firstWhere = true;
                    if (substr($sortVar,1,1) ==':' || substr($sortVar,1,1) =='=')
                        $sql->sql.= substr($sortVar,0,1).'__';
                    $sql->sql .= $query->getReportMetaRecord($sortVar)->getSelName();
                    $sql->sql .= " as sortValue, stufe ";
                    for ($i = 0; $i < $query->repRows->size(); $i++) {
                        if (!$query->repRows->getPos($i)->webLinkColumn){
                            $var = $query->repRows->getPos($i)->getName();
                            if (isValid($var)) {
                                $mr = $query->getReportMetaRecord($var);
                                $sql->sql .= ", ";
                                $sql->sql .= $mr->getSelName();                            
                            }
                        }
                    }
                    $sql->sql .= "\n from WSR_";
                    $sql->sql .= $baseTable;
                    if ( $query->repColumns->size() > 0) {                      
                        $sql->addSel('colgroup = ', $colGroup);
                        if ($colGroup != 99  && isValid($sortColGroup)) {
                            $mrc = $query->getReportMetaRecord($sortColVar);
                            $vn = $mrc->getSelName();
                          
                            if (substr($sortColVar,1,1)==':') $vn = substr($sortColVar,0,1).'__'.$vn;
                            $sql->addSel($vn.' = ',$sortColValue);
                        }
                    }
                    if ($query->sort_mode == SortOptions::SORTLASTGROUP) {
                        $sql->addSel('stufe = ', $query->repRows->size());
                    }
                    
                    $sql->sql .= " )  sort0\n on ";
                }
                $sql->sql .= " base.stufe = sort0.stufe \nand (base.";
                $vn = $query->getReportMetaRecord($query->repRows->getPos(0)->getName())->getSelName();
                $sql->sql .= $vn;
                $sql->sql .= " = sort0.";
                $sql->sql .= $vn;
                $sql->sql .= " or (base.";
                $sql->sql .= $vn;
                $sql->sql .= " is null and sort0.";
                $sql->sql .= $vn;
                $sql->sql .= " is null ))";
                
                for ($i = 1; $i < $query->repRows->size(); $i++) {
                    if (!$query->repRows->getPos($i)->webLinkColumn){
                        $vn = $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                        $sql->sql .= "\n and (base.";
                        $sql->sql .= $vn;
                        $sql->sql .= " = sort0.";
                        $sql->sql .= $vn;
                        $sql->sql .= " or (base.";
                        $sql->sql .= $vn;
                        $sql->sql .= " is null and sort0.";
                        $sql->sql .= $vn;
                        $sql->sql .= " is null ))";
                    }
                }
                if ($query->sort_mode == SortOptions::SORTTABLE){
                    $sql->addSel('base.stufe in (0, ', $query->repRows->size());
                    $sql->sql .= ')';
                }
                $sortNachWert = true;
            }else if ($query->sort_mode == SortOptions::SORTSPECIAL || $query->sort_mode == SortOptions::SORTSPECIALSZ) {
                for ($s=0; $s < $query->repRows->size(); $s++) {
                    if (!$query->repRows->getPos($s)->webLinkColumn){
                        if (isset($mysql)){
                            $sql->sql .= "\nleft join WSR_sorter".$s." sort".$s."\non ";
                        }else{
                            
                        $sql->sql .= "\nleft join (select ";
                        $sql->firstWhere = true;
                        if ( $query->sort_mode == SortOptions::SORTSPECIALSZ
                            || $s == $query->repRows->size() -1
                            || $query->repRows->getPos($s+1)->summenZeilen ){
                                // baseTable verwenden
                                if (substr($sortVar,1,1) ==':' || substr($sortVar,1,1) =='=')
                                    $sql->sql.= substr($sortVar,0,1).'__';
                                $sql->sql .= $query->getReportMetaRecord($sortVar)->getSelName();
                                $sql->sql .= " as sortValue";
                                for ($i = 0; $i <= $s; $i++) {
                                    if (!$query->repRows->getPos($i)->webLinkColumn){     
                                        $mr = $query->getReportMetaRecord($query->repRows->getPos($i)->getName());
                                        if ($mr != null){
                                            $sql->sql .= ", ";
                                            $sql->sql .= $mr->getSelName();
                                        }
                                    }
                                }
                                $sql->sql .= "\n from WSR_";
                                $sql->sql .= $baseTable;
                                $sql->addSel("Stufe = ", ($s+1)) ;
                                if ( $query->repColumns->size() > 0) $sql->addSel("colgroup = ", $colGroup);
                                
                                $s2 = $s;
                                while ($s2 < $query->repRows->size()-1 && $query->repRows->getPos($s2+1)->webLinkColumn){
                                    $s2++;
                                }
                                
                                if ($s2 < $query->repRows->size() -1) $sql->addWhere("S".($s2+1));
                                
                                if ($colGroup != 99 && isValid($sortColGroup)) {
                                    $mrc = $query->getReportMetaRecord($sortColVar);
                                    $vn = $mrc->getSelName();
                                    
                                    if (substr($sortColVar,1,1)==':') $vn = substr($sortColVar,0,1).'__'.$vn;
                                    $sql->addSel($vn.' = ',$sortColValue);
                                }
                        }else {
                            // Summen-Werte neu ermitteln
                            if (substr($sortVar,1,1) ==':' || substr($sortVar,1,1) =='=')
                                $sql->sql.= substr($sortVar,0,1).'__';
                            $sql->sql .= $query->getReportMetaRecord($sortVar)->getSelName();
                            $sql->sql .= " as sortValue";
                            for ($i = 0; $i <= $s; $i++) 
                                if (!$query->repRows->getPos($i)->webLinkColumn){
                                $sql->sql .= ", ";
                                $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                            }
                            $sql->sql .= "\n from WSR_";
                            $sql->sql .= $baseTable;
                            if (isValid($sortColGroup) && isValid($sortColValue)) {
                                $mrc = $query->getReportMetaRecord($sortColVar);
                                $vn = $mrc->getSelName();
                                
                                if (substr($sortColVar,1,1)==':') $vn = substr($sortColVar,0,1).'__'.$vn;
                                $sql->addSel($vn.' = ',$sortColValue);
                                $rc = $query->repColumns->getPos($sortColGroup);
                                if ($rc->filter != null) {
                                    $sql ->addFilterSQL($rc->filter,$rc,null);
                                }
                            }
                            $sql->sql .= "\ngroup by ";
                            $mrr = $query->getReportMetaRecord($query->repRows->getPos(0)->getName());
                            $sql->sql .= $mrr->getSelName();
                            
                            for ($i = 1; $i <= $s; $i++) 
                                if (!$query->repRows->getPos($i)->webLinkColumn){
                                $sql->sql .= ", ";
                                $mrr = $query->getReportMetaRecord($query->repRows->getPos($i)->getName());
                                $sql->sql .= $mrr->getSelName();                            
                            }
                        }
                        $sql->sql .= " )sort".$s." on ";
                        }
                        $sql->sql .= "\n (base.";
                        $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos(0)->getName())->getSelName();
                        $sql->sql .= " = sort".$s.".";
                        $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos(0)->getName())->getSelName();
                        $sql->sql .= " or (base.";
                        $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos(0)->getName())->getSelName();
                        $sql->sql .= " is null and sort".$s.".";
                        $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos(0)->getName())->getSelName();
                        $sql->sql .= " is null ))";
                        
                        for ($i = 1; $i <= $s; $i++) {
                            if (! $query->repRows->getPos($i)->webLinkColumn){
                            $sql->sql .= "\n and (base.";
                            $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                            $sql->sql .= " = sort".$s.".";
                            $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                            $sql->sql .= " or (base.";
                            $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                            $sql->sql .= " is null and sort".$s.".";
                            $sql->sql .= $query->getReportMetaRecord($query->repRows->getPos($i)->getName())->getSelName();
                            $sql->sql .= " is null ))";
                            }
                        }
                    }
                }
                $sortNachWert = true;
            }else {
                addMeldung($sql,"Unbekannter query->sort_mode = ".$query->sort_mode);
            }
            
        }
        /* NOCH MACHEN !!!
         // Werte-Filter
         if (! filter.isEmpty()) {
         Enumeration<String> keys = filter.keys();
         while (keys.hasMoreElements()) {
         String key = keys.nextElement();
         if (filter.get(key) != null && isValueFilter(key)) {
         String[] fiv =  filter.get(key);
         if ($first) $sql->sql .= "\nwhere (";
         else $sql->sql .= "\n and (";
         $first = false;
         $sql->sql .= createFilterSQL(key, fiv, false, true, "base"));
         if (isValid(fiv[3])){
         int pos = Integer.parseInt(fiv[3]);
         if (pos > 0) {
         if (pos <= $query->repRows->size()) $sql->sql .= " or base.S".(pos-1)." > 0";
         else $sql->sql .= " or base.S".($query->repRows->size()-1)." > 0";
         }
         }
         $sql->sql .= ")";
         }
         }
         }
         **/
        
        
        // Sortieren
            $sql->sql .= $sortOrderBy;
            return $sql;
}




    