<?php
/**
 * Projekt: WS-Reporter
 *          Reportdefinition einlesen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    10.12.2021
 * 
 * Status: Liest alte Report-Version
 *
 */
require_once './includes/loadClassDef.inc';
require_once './includes/funktionen.inc';
function loadReportV1($title){
    global $query, $meldungen;
    require_once './phpsec/createDBH.inc';    
    $sql = new SqlAbfrage('WSR');
    $sql->sql = "select * from reportDefinitionen";
    $sql->addSel("reportTitle = ", $title);
    $sql->sql.=" order by SortNr";
    $stmt = $sql->execute();
    if ($stmt === false){
        http_response_code(425);
        echo 'Request:<br><pre>';
        echo var_dump($_REQUEST);
        echo '</pre><br>DB-Fehler:<br><pre>';
        echo var_dump($sql->errorInfo());
        echo '</pre>';
        die();
    }
    $z = $stmt->fetchObject();
    if ($z === false){
        return false;
    }
    
    $query = new Query($z->reportDef7);
    if ((array_key_exists('action', $_REQUEST) && $_REQUEST['action'] == 'reload')) {
        $query->loadSaveAddFilter = true;
    }
    loadClassDef($query);
    /*
    if (!isValid($query->title)) $query->title = $query->defaultTitle;
    if($query->repRows == null) $query->repRows= new Vector();
    if($query->repValues == null) $query->repValues= new Vector();
    if($query->repColumns == null) $query->repColumns= new Vector();
    if($query->repFilter == null) $query->repFilter= new Vector();
    if($query->repTables == null) $query->repTables= new Vector();
    if($query->addQueries == null) $query->addQueries= new Vector();
    */
    $query->newReport();
    $query->rLanguage = 'de';
    
//    error_log('Vorher:'.print_r($query->repFilter,true));
    
    $rfv = $query->repFilter;
    for($i = $rfv->size(); $i > 0; ){
        $i --;
        $rf = $rfv->getPos($i);
        if (! $query->metaData[$rf->getName()]->hidden) $rfv->removePos($i);
    }
//    error_log('Nachher:'.print_r($query->repFilter,true));
    
    
    $query->sort_mode = $z->reportDef1;
    if (isValid($z->reportDef2)) $query->repGeldFaktor = $z->reportDef2;
    if (isValid($z->reportDef3)) {
        if (strpos($z->reportDef3, 'fixierte Label') !== false) $query->fix_labelColumns = true;
        if (strpos($z->reportDef3, 'drop Summen') !== false) $query->dropSummen = true;
        if (strpos($z->reportDef3, 'Folder offen') !== false) $query->folderMode = Query::FOLDEROFFEN;
        else
        if (strpos($z->reportDef3, 'Folder zu') !== false) $query->folderMode = Query::FOLDERZU;
        else
        if (strpos($z->reportDef3, 'Folder 1.Stufe') !== false) $query->folderMode = Query::FOLDERZU1;
        else
        if (strpos($z->reportDef3, 'Folder 2 Stufen') !== false) $query->folderMode = Query::FOLDERZU2;
        else $query->folderMode = Query::KEINE;
    }
    if (isValid($z->reportDef4)) {
        $query->values_in_columns = true;
        if (strpos($z->reportDef4, 'WSFSummen') !== false) $query->valueColumnFolderMode = 'WSFSummen';
        else
        if (strpos($z->reportDef4, 'WSFHeader') !== false) $query->valueColumnFolderMode = 'WSFHeader';        
    }
    if (isValid($z->reportDef5)) $query->chart_size = $z->reportDef5;
    if (isValid($z->reportDef6)) $query->display_charts = $z->reportDef6;
    if (isValid($z->reportDef7)) if ($query->class_table != $z->reportDef7){
        http_response_code(400);
        echo 'Datenquelle des Reports '.$z->reportDef7.' passt nicht zur aktuellen Datnquelle '.$query->class_table;
        die();
    }
    if (isValid($z->reportDef8) && strpos($z->reportDef8,'Language:') === 0){
        $reportVersion = 2;
        $aSprachen = explode(' ', substr($z->reportDef8, strlen('Language:')));
        $query->rLanguage = $aSprachen[0];
        $query->rTranslations = array();
        for ($i = 1; $i < count($aSprachen); $i++)
            if ($aSprachen[$i] != $aSprachen[0])
                $query->rTranslations[] = $aSprachen[$i];
       if ($query->currentLanguage != $query->rLanguage 
           && false === array_search($query->currentLanguage, $query->rTranslations)){
               $meldungen.="\n<br>ACHTUNG: Für die aktuelle Sprache ".$query->currentLanguage
                    .' gibt es keine Übersetzung des Reports.';
       }
    }else $reportVersion = 1;

    if (isValid($z->reportDef9) && strpos($z->reportDef9, 'H:') === 0 ) {
        $histEntries = explode(':', substr($z->reportDef9,2));
//        error_log(print_r($histEntries,true));
        foreach ($histEntries as $hEntry){
            $hEntryArray = explode('.',$hEntry);
//            error_log(print_r($hEntryArray,true));
            if (isValid($hEntryArray[0])){
                if (isValid($hEntryArray[1])){
                    if ($hEntryArray[0] == 0){
                        $query->historyOption = 1* $hEntryArray[1];
                        if (array_key_exists(2, $hEntryArray)){
                            $query->historyDate = $hEntryArray[2];
                        }
                    }else{
                        $dim = $query->dimensions[$hEntryArray[0]];
                        if ($dim == null){
                            error_log("Fehler beim Einlesen der Historisierung Nr {$hEntryArray[0]} für die Basisquelle ".$z->reportDef9);
                        }else {
                            $dim->historyOption = 1* $hEntryArray[1];
                            if (array_key_exists(2, $hEntryArray)){
                                $dim->historyDate = $hEntryArray[2];
                            }
                        }
                    }
                }
            }
        }
    }
    $query->title = $title;
    
    
    $tmeta = array();
    $query->kommentar = '';
    while ($z = $stmt->fetchObject()){
        $joinVar = false;    
        if (isValid($z->reportDef1)){
            $name = $z->reportDef1;
            if (substr($name, 1,1) == ":") $joinVar = true;
            if (substr($name, 1,1) == "=") $joinVar = true;
            if ($joinVar) {
                $name = substr($name,0,2).substr($name,3); // Blank entfernen
                if ($z->SatzTyp <= 4) $tmeta[$name] = substr($name, 1,1);
            }
            
        switch ($z->SatzTyp) {
            // Zeile
            case 1:
                    $row = new RepRow($name);                    
                    $query->repRows->put($row);
                    if (isValid($z->reportDef9) && substr($z->reportDef9,0,4) == 'WLC:'){
                        $row->webLinkColumn = true;
                        $row->web_link_value = substr($z->reportDef9,4);
                        if (isValid($z->reportDef2)) $row->web_link_title = $z->reportDef2;
                    }else{
                        if (isValid($z->reportDef2)) $row->summenZeilen = true;                        
                    }
                    if (isValid($z->reportDef3))
                        if ($z->reportDef3 == 'ascending') $row->sort = Sort::SORTANZEIGE;
                        else
                        if ($z->reportDef3 == 'descending') $row->sort = Sort::SORTANZEIGEDESC;
                    if (isValid($z->reportDef4)) $row->web_link = $z->reportDef4;
                    if (isValid($z->reportDef5)) $row->webCopyFilter = true;
                    if (isValid($z->reportDef6)) $row->fixedTableColumns = true;
                    if (isValid($z->reportDef7)) $row->web_title = $z->reportDef7;
                    if (isValid($z->reportDef8)) $row->web_target = $z->reportDef8;
                    break;
            // Wert
            case 2:
                $row = new RepValue($name);
                    $query->repValues->put($row);
                    if (isValid($z->reportDef2)) $row->dispVal = $z->reportDef2;
                    if (isValid($z->reportDef3)) $row->skalierung = 1 * $z->reportDef3;
                    if (isValid($z->reportDef4)) $row->keineSummen = true;
                    if (isValid($z->reportDef5)) $row->base = $z->reportDef5;
                    if (isValid($z->reportDef6)) $row->label = $z->reportDef6;
                    if (isValid($z->reportDef7)) $row->backgroundColor = $z->reportDef7;
                    break;
            // Filter        
            case 3:
                if (substr($name,0,1) == '#') $row = new WindowFilter($name);
                else $row = new RepFilter($name);
                    $query->repFilter->put($row);
                    if (isValid($z->reportDef2)) $row->fs_von = $z->reportDef2;
                    if (isValid($z->reportDef3)) $row->fs_bis = $z->reportDef3;
                    if (isValid($z->reportDef4)) $row->inTitelZeile = $z->reportDef4;
                    if (isValid($z->reportDef5)) $row->abSpalte = $z->reportDef5;
                    if (isValid($z->reportDef6)){
                        if ( strpos($z->reportDef6, "selectorImOutput") ) $row->selectorImOutput = true;
                        if ( strpos($z->reportDef6, "ausschlussListe") ) $row->ausschlussListe = true;
                        if ( strpos($z->reportDef6, "inTitelMitLabel") ) $row->inTitelMitLabel = true;
                    }
                    if (isValid($z->reportDef7)) $row->reportCopyFilter = true;
                    if (isValid($z->reportDef8)) $row->doNotTake = true;
                    $mr = $query->getReportMetaRecord($name);
                    if ($reportVersion == 1 && $mr != null){
                            if (isValid($row->fs_von)) $row->fs_von = $mr->reformatValue($row->fs_von);
                            if (isValid($row->fs_bis)) $row->fs_bis = $mr->reformatValue($row->fs_bis);
                    }
                    if ($mr != null && $mr->filterType == MetaRecord::SONDERFILTERSEL){
                        $row->fs_von = qTranslate::translateReport($query, $row->fs_von);
                    }
                    break;
            // Filter-Listen: ausgewählte Werte        
            case 53:
                $repFilter = $query->repFilter->get($name);
                if ($repFilter->fs_list == null) $repFilter->fs_list = array();
                if (isValid($z->reportDef2)) $repFilter->fs_list[] = $z->reportDef2;
                if (isValid($z->reportDef3)) $repFilter->fs_list[] = $z->reportDef3;
                if (isValid($z->reportDef4)) $repFilter->fs_list[] = $z->reportDef4;
                if (isValid($z->reportDef5)) $repFilter->fs_list[] = $z->reportDef5;
                if (isValid($z->reportDef6)) $repFilter->fs_list[] = $z->reportDef6;
                if (isValid($z->reportDef7)) $repFilter->fs_list[] = $z->reportDef7;
                if (isValid($z->reportDef8)) $repFilter->fs_list[] = $z->reportDef8;
                if (isValid($z->reportDef9)) $repFilter->fs_list[] = $z->reportDef9;
                break;
                // Value-Filter, für Aufruf aus Report-Editor        
            case 20:
                if (isValid($z->reportDef2)) $query->repWerteBasis = $z->reportDef2;
                if (isValid($z->reportDef3)) $query->repWerteAuswahlTitel = $z->reportDef3;
                if (isValid($z->reportDef4)) $query->repWerteAuswahlLabel = true;     
                if (isValid($z->reportDef5)) $query->repWerteAuswahlSelector = true;
                break;
                // Zusatzfilter, für Aufruf aus Report-Editor
            case 23:
                if ($query->loadSaveAddFilter){
                    $row = new RepFilter($name);
//                    echo "Load Addfilter $name<br>";
                    $query->addFilter->put($row);
                    if (isValid($z->reportDef2)) $row->fs_von = $z->reportDef2;
                    if (isValid($z->reportDef3)) $row->fs_bis = $z->reportDef3;
                    if (isValid($z->reportDef4)) $row->inTitelZeile = $z->reportDef4;
                    if (isValid($z->reportDef5)) $row->abSpalte = $z->reportDef5;
                    if (isValid($z->reportDef6)) $row->selectorImOutput = true;
                    if ($reportVersion == 1){
                        $mr = $query->getReportMetaRecord($name);
                        if (isValid($row->fs_von)) $row->fs_von = $mr->reformatValue($row->fs_von);
                        if (isValid($row->fs_bis)) $row->fs_bis = $mr->reformatValue($row->fs_bis);
                    }                    
                }
                break;
                // Zusatzfilter-Listen: ausgewählte Werte
            case 54:
                if ($query->loadSaveAddFilter){
                    $repFilter = $query->addFilter->get($name);
                    if ($repFilter->fs_list == null) $repFilter->fs_list = array();
                    if (isValid($z->reportDef2)) $repFilter->fs_list[] = $z->reportDef2;
                    if (isValid($z->reportDef3)) $repFilter->fs_list[] = $z->reportDef3;
                    if (isValid($z->reportDef4)) $repFilter->fs_list[] = $z->reportDef4;
                    if (isValid($z->reportDef5)) $repFilter->fs_list[] = $z->reportDef5;
                    if (isValid($z->reportDef6)) $repFilter->fs_list[] = $z->reportDef6;
                    if (isValid($z->reportDef7)) $repFilter->fs_list[] = $z->reportDef7;
                    if (isValid($z->reportDef8)) $repFilter->fs_list[] = $z->reportDef8;
                    if (isValid($z->reportDef9)) $repFilter->fs_list[] = $z->reportDef9;
                }
                break;
                // Spalte    
            case 4:
                    $narr = explode(' ', $name);                    
                    $row = new RepColumn($narr[0]);
                    if (count($narr) > 1 && $narr[1] = "descending") $row->sort = Sort::SORTINTERNDESC;
                    $query->repColumns->put($row);
                    if (isValid($z->reportDef2)) $row->label = $z->reportDef2;
                    if (isValid($z->reportDef3)) $row->val_name = $z->reportDef3;
                    if (isValid($z->reportDef4)) $row->filter = $z->reportDef4;
                    if (isValid($z->reportDef5)) $row->fs_von = $z->reportDef5;
                    if (isValid($z->reportDef6)) $row->fs_bis = $z->reportDef6;
                    if (isValid($z->reportDef7)) $row->web_link = $z->reportDef7;
                    if ($reportVersion == 1){
                        $mr = $query->getReportMetaRecord($row->filter);
                        if (isValid($row->fs_von)) $row->fs_von = $mr->reformatValue($row->fs_von);
                        if (isValid($row->fs_bis)) $row->fs_bis = $mr->reformatValue($row->fs_bis);
                    }
                    
                    if (isValid($z->reportDef8)){
                        $linkOptions = $z->reportDef8;
                        if (strpos($linkOptions, ':')){
                            $row->web_title = substr($linkOptions, strpos($linkOptions, ':')+1);
                            $linkOptions =  substr($linkOptions, 0, strpos($linkOptions, ':'));
                        }
                        if (strpos($linkOptions, '|')){
                            $row->web_target = substr($linkOptions, strpos($linkOptions, '|')+1);
                            $linkOptions =  substr($linkOptions, 0, strpos($linkOptions, '|'));
                        }
                        if (strpos($linkOptions,'copy Filter') === false)
                            $row->reportCopyFilter = false;
                            else $row->reportCopyFilter = true;
                        if (strpos($linkOptions,'Link in Values') === false)
                            $row->reportLinkInValues = false;
                            else $row->reportLinkInValues = true;
                        if (strpos($linkOptions,'keineSummen') === false)
                            $row->keineSummen = false;
                            else $row->keineSummen = true;
                    }
                    if (isValid($z->reportDef9)){
                        $r9 = explode(' ', $z->reportDef9);
                        $row->displayOption = $r9[0];
                        if (count($r9) > 1) $row->skalierung = 1 * $r9[1];
                    }
                    if ($joinVar) {
                        // Blanks in Namen entfernen
                        if (isValid($row->val_name)){                            
                            $row->val_name = substr($row->val_name,0,2).substr($row->val_name,3);
                            $tmeta[$row->val_name] = substr($row->val_name, 1,1);
                        }
                        if (isValid($row->filter)){
                            $row->filter = substr($row->filter,0,2).substr($row->filter,3);
                            $tmeta[$row->filter] = substr($row->filter, 1,1);
                        }
                        
                    }
                break;
            }
        } // Ende Spalten
        
        // Globale Optionen ohne Spalten-Namen
        switch ($z->SatzTyp) {
            // Kommentar
            case 10:
                if ($z->reportDef1 != null) $query->kommentar .= $z->reportDef1."\n";
                if ($z->reportDef2 != null) $query->kommentar .= $z->reportDef2."\n";
                if ($z->reportDef3 != null) $query->kommentar .= $z->reportDef3."\n";
                if ($z->reportDef4 != null) $query->kommentar .= $z->reportDef4."\n";
                if ($z->reportDef5 != null) $query->kommentar .= $z->reportDef5."\n";
                if ($z->reportDef6 != null) $query->kommentar .= $z->reportDef6."\n";
                if ($z->reportDef7 != null) $query->kommentar .= $z->reportDef7."\n";
                if ($z->reportDef8 != null) $query->kommentar .= $z->reportDef8."\n";
                if ($z->reportDef9 != null) $query->kommentar .= $z->reportDef9."\n";
        
                break;
            case 11:
                // Report-Verweise 
                if (isValid($z->reportDef1)){
                    $name = $z->reportDef1;
                    $rep = new RepReport($name);
                    if (isValid($z->reportDef2)) $rep->kommentar = $z->reportDef2;
                    if (isValid($z->reportDef3)) $rep->copyFilter = true;
                    $query->repReports->put($rep);
                }
                break;
            default:
                if ($z->SatzTyp >= 100){
                    // verknüpfte Datenquellen
                    error_log('Verknüpfte Datenquelle '.$z->reportDef2);
                    $aq = $query->addQueries->get($z->reportDef2);
                    if ($aq == null){ 
                        $meldungen .= "\n<br>Fehler: Der Report verlangt die verknüpfte Datenquelle <b>".$z->reportDef2
                        .'</b> als <b>'.$z->reportDef3.'</b> ('.$z->reportDef4.').<br>Sie ist in der Basis-Datenquelle nicht deklariert.';
//                        return;
                    }else{
                        $jt = new JoinedTable($aq);
                        $jt->kennzeichen = $z->reportDef3;
                        $jt->prefix = $z->reportDef4;
                        $jt->joinType = $z->reportDef5;
                        if ($jt->query == null || $jt->query->phys_table == null){
                            $meldungen .= "\n<br>Fehler beim Lesen der verknüpften Datenquelle <b>".$z->reportDef2
                            .'</b> als <b>'.$z->reportDef3.'</b> ('.$z->reportDef4.')';
                        }
                        $query->repTables->put($jt);
                        error_log('Verknüpfte Datenquelle hat Kennzeichen '.$jt->kennzeichen);
                        if (isValid($z->reportDef9) && strpos($z->reportDef9, 'H:') === 0 ) {
                            $histEntries = explode(':', substr($z->reportDef9,2));
//                            error_log(print_r($histEntries,true));
                            foreach ($histEntries as $hEntry){
                                $hEntryArray = explode('.',$hEntry);
//                                error_log(print_r($hEntryArray,true));
                                if (isValid($hEntryArray[0])){
                                    if (isValid($hEntryArray[1])){
                                        $jt->dimHistoryOption[1*$hEntryArray[0]] = 1* $hEntryArray[1];
                                        if (array_key_exists(2, $hEntryArray)){
                                            $jt->dimHistoryDate[1*$hEntryArray[0]] = $hEntryArray[2];
                                        }
                                    }
                                }
                            }
                        }
                    }
                }                
            break;
        }
    }
    
    $stmt->closeCursor();
    foreach ($tmeta as $tn => $tk){
        $k = substr($tn, 0,1);
        $rt = $query->repTables->get($k);
        if ($rt == null){
            echo 'FEHLER Verknüpfte Datenquelle '.$k.' ist nicht spezifiziert.<br>';
        }
    }
    
    $query->titleModified = false;
    rechteAnwenden();
    return true;
}