<?php
/**
 * Projekt: WS-Reporter
 *          Speichern einer Report-Definition
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    23.02.2023
 *
 */
global $dbh, $query;

require_once './phpsec/createDBH.inc';
if (!dqAdminRechte()){
    http_response_code(425);
    echo "Für diese Operation sind die Administrator-Rechte \nfür die Datenquelle erforderlich";
    die;
}
if (isset($_REQUEST['saveAsProd'])) {
    require './phpsec/createProdDBH.inc';
}
require_once './includes/classes.inc';
require_once './includes/qTranslate.inc';

$sprache = 'de';
if (array_key_exists('Sprache', $_SESSION)) $sprache = substr($_SESSION['Sprache'],0,2);
if ($query->reportImported) $sprache = $query->rLanguage;

error_log('save Report Def. ID='.print_r($query->reportID,true));

if ($query->reportID <= 0){
 // Neuen Eintrag anlegen
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'insert into reportTitle set reportVersion = 2';
    // GGf. eingelesen Übersetzungen aus Import stehen lassen
        switch ($sprache){
            case 'de': $query->title_de = $query->title;
            break;
            case 'en': $query->title_en = $query->title;
            break;
            case 'fr': $query->title_fr = $query->title;
            break;
        }
        if (!isset($query->title_de)) $query->title_de = $query->title;
        if (!isset($query->title_en)) $query->title_en = $query->title;
        if (!isset($query->title_fr)) $query->title_fr = $query->title;
        
        $sql->addKlausel(',reportTitle_de =', $query->title_de);
        $sql->addKlausel(',reportTitle_en =', $query->title_en);
        $sql->addKlausel(',reportTitle_fr =', $query->title_fr);
        
    if ($sql->execute() === false){
        // Hier wird der SQLSTATE Error-Code abgefragt, statt des mariaDB-codes in [1]
        // weil vermutet wird, dass er allgemeiner verwendet wird
        if ($sql->errorInfo()[0] == 23000){
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'select * from reportTitle ';
            $sql->addSel('reportTitle_de =', $query->title_de);
            $sql->addKlausel('or reportTitle_en =', $query->title_en);
            $sql->addKlausel('or reportTitle_fr =', $query->title_fr);
            $stmt = $sql->execute();
//            error_log('SQL: '.$sql->sql);
            if ($stmt === false){
                http_response_code(425);
                echo 'FEHLER bei select from reportTitle';
                echo $stmt->errorInfo()[2];
                die();
            }
            $z = $stmt->fetch(PDO::FETCH_ASSOC);
            if (array_key_exists('bestaetigt', $_REQUEST)){
                // Eintrag überschreiben
                $query->reportID = $z['reportID'];
                $sql = new SqlAbfrage('WSR');
                $sql->sql = 'update reportTitle set reportVersion = 2';
                
                $sql->addKlausel(',reportTitle_'.$sprache.' =', $query->title);
                
                $sql->addSel('reportID = ', $query->reportID);
                $stmt = $sql->execute();
                if ($stmt === false){
                    http_response_code(425);
                    echo 'FEHLER bei update reportTitle';
                    echo $sql->errorInfo()[2];
                    error_log('FEHLER bei update reportTitle');
                    error_log(print_r($sql,true));
                    die();
                }
                error_log('HINWEIS: Beim Überschreiben von Reports wird nur der Name'
                    .' in der aktuellen Sprache ersetzt.');
            }else{
                http_response_code(201);
                echo WSRTranslate::translate('Report').' '.$z['reportTitle_'.$sprache]
                  .' '.WSRTranslate::translate('überschreiben?');
//                error_log( 'Report '.$z['reportTitle_'.$sprache].' überschreiben?');
                die();                
            }
        }else{        
            error_log('Sql-Error-Info bei Insert Title '.print_r($sql->errorInfo(),true));
            http_response_code(425);
            echo 'FEHLER bei insert Report Title '.$query->title;
            echo $stmt->errorInfo()[2];
            die();
        }
    }else{
        // Neuer Eintrag war erfolgreich
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'select last_insert_id() as reportID';
        $stmt = $sql->execute();
        if ($stmt === false){
            http_response_code(425);
            echo 'FEHLER bei select last_insert_id()';
            echo $stmt->errorInfo()[2];
            die();
        }
        $z = $stmt->fetchObject();
        $query->reportID = $z->reportID;
        
        // Log für Neuen Eintrag
        $logSql = new SqlAbfrage('WSR');
        $logSql->sql = 'insert into wsrAdminReport set timeCreated = sysdate()';
        $logSql->addKlausel(', userCreated = ', $_SESSION['Name']);
        $logSql->addKlausel(', reportID = ', $query->reportID);
        $erg = $logSql->executeE(false);
        if ($erg === false || $erg->rowCount() < 1 ){
            error_log('Fehler beim Insert wsrAdminReport: '. $logSql->errorInfo()[2]);
        }
    }
}else{
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select * from reportTitle';
    $sql->addSel('reportID =', $query->reportID);
    $stmt = $sql->execute();
    if ($stmt === false){
        http_response_code(425);
        echo 'FEHLER bei select Report Title';
        echo $stmt->errorInfo()[2];
        die();
    }
    $z = $stmt->fetch(PDO::FETCH_ASSOC);
    if ($z === false){
        http_response_code(425);
        echo 'FEHLER bei select Report Title für '.$query->reportID;
        echo $stmt->errorInfo()[2];
        die();
        
    }
    $update = false;
    if (!isValid($z['reportTitle_de'])) $update = true;
    if (!isValid($z['reportTitle_en'])) $update = true;
    if (!isValid($z['reportTitle_fr'])) $update = true;
    if ($z['reportTitle_'.$sprache] != $query->title) {
        $z['reportTitle_'.$sprache] = $query->title;
        $update = true;
    }
    if ($z['reportVersion'] <2 ) $update = true;
    $stmt->closeCursor();
    if ($update){
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'update reportTitle set reportVersion = 2';
        $sql->addKlausel(', reportTitle_'.$sprache.' = ', $query->title);
        if ($sprache != 'de' && !isValid($z['reportTitle_de']))
            $sql->addKlausel(', reportTitle_de = ', $query->title);
        if ($sprache != 'en' && !isValid($z['reportTitle_en']))
            $sql->addKlausel(', reportTitle_en = ', $query->title);
        if ($sprache != 'fr' && !isValid($z['reportTitle_fr']))
            $sql->addKlausel(', reportTitle_fr = ', $query->title);
        $sql->addSel('reportID = ', $query->reportID);
        if (! $sql->execute() ){
            if ($sql->errorInfo()[0] = 23000){
                http_response_code(425);
                echo WSRTranslate::translate('Unter diesem Namen gibt es bereits eine Report-Definition.');
                die();
            }
            http_response_code(425);
            echo "Fehler beim Update von reportTitle:\n";
            echo $sql->errorInfo()[2];
            die();
        }
    }
}
error_log('Save Report ID '.$query->reportID.': '.$query->title);

// Attribute 
$sql = new SqlAbfrage('WSR');
$sql->sql = 'delete from reportAttributes ';
$sql->addSel('reportID = ', $query->reportID);
if (! $sql->execute() ){
    echo 'Fehler beim Löschen des Eintrags in reportAttributes';
    echo $sql->errorInfo()[2];
    die();
}
$sql = new SqlAbfrage('WSR');
$sql->sql = 'insert into reportAttributes set ';
$sql->addKlausel('reportID = ', $query->reportID);
$sql->addKlausel(',table_name = ', $query->class_table);
if (isset($query->repGeldFaktor) && $query->repGeldFaktor > 0)
    $sql->addKlausel(',repGeldFaktor = ', $query->repGeldFaktor);
else $sql->sql .= ',repGeldFaktor =  1';
$sql->addKlausel(',sort_mode = ', $query->sort_mode);
$sql->addKlausel(',fix_labelColumns = ', bool($query->fix_labelColumns));
$sql->addKlausel(',values_in_columns = ', bool($query->values_in_columns));
$sql->addKlausel(',folderMode = ', $query->folderMode);
$sql->addKlausel(',valueColumnFolderMode = ', $query->valueColumnFolderMode);
$sql->addKlausel(',chart_size = ', $query->chart_size);
$sql->addKlausel(',display_charts = ', $query->display_charts);
$sql->addKlausel(',dropSummen = ', bool($query->dropSummen));

$historyOption = '';
if ($query->histType > 0){
    $historyOption .= '0.'.$query->historyOption;
    if (isValid($query->historyDate))$historyOption .= '.'.$query->historyDate;
    $historyOption .=':';
}
foreach($query->dimensions as $dimNr => $dim){
    if (false) $dim = new Dimension(null);
//    error_log("Dim: $dimNr - ".$dim->dimName);
    if (isValid($dim->historyOption) &&  $query->reportHasDim('base', $dimNr)){
        $historyOption .= $dimNr.'.'.$dim->historyOption;
        if (isValid($dim->historyDate)) $historyOption .='.'.$dim->historyDate;
        $historyOption .=':';
//        error_log('save '.$dim->historyOption);
    }
}
$sql->addKlausel(',historyOptions = ', $historyOption); 
$sql->addKlausel(',rLanguage  = ', $query->rLanguage );
$rTranslations = '';
foreach ($query->rTranslations as $tl){
    if ($tl != $query->rLanguage) $rTranslations .= ' '.$tl;
}
$sql->addKlausel(',rTranslations = ', $rTranslations);
$sql->addKlausel(',repWerteBasis = ', $query->repWerteBasis);
$sql->addKlausel(',repWerteAuswahlTitel = ', $query->repWerteAuswahlTitel);
$sql->addKlausel(',repWerteAuswahlLabel = ', bool($query->repWerteAuswahlLabel));
$sql->addKlausel(',repWerteAuswahlSelector = ', bool($query->repWerteAuswahlSelector));
if (! $sql->execute() ){
    echo WSRTranslate::translate('Fehler beim Eintragen in').' reportAttributes: ';
    echo $sql->errorInfo()[2];
//    error_log(print_r($sql,true));
    die();
}

insertReportComment($query->reportID,$query->rLanguage,$query->kommentar);
if ($query->reportImported && isset($query->description_imported)){
    foreach ($query->description_imported as $lang => $comment){
        if (isValid($comment))
            insertReportComment($query->reportID,$lang,$comment);
    }
}

insertReportHints($query->reportID,$query->rLanguage,$query->report_hints);
if ($query->reportImported && isset($query->hints_imported)){
    foreach ($query->hints_imported as $lang => $comment){
        if (isValid($comment))
            insertReportHints($query->reportID,$lang,$comment);
    }
}

    
 // ENDE Speichern des Kommentars




$query->repRows->save($query->reportID);
$query->repColumns->save($query->reportID);
$query->repValues->save($query->reportID);
$query->repReports->save($query->reportID);

// Speicher der Filter - funktioniert nicht über Vector->save(),  
// Zusätzlich die Filter-Liste speichern
$query->repFilter->save($query->reportID);
// Filter-Listen 
$sql = new SqlAbfrage('WSR');
$sql->sql = 'delete from reportFilterLists';
$sql->addSel('reportID = ',$query->reportID);

if ($sql->execute() === false){
    http_response_code(425);
    echo "\nFehler bei Delete Filter-Lists";
    echo "\n".$sql->errorInfo()[2];
    die();
}
$stmt = $dbh->prepare('insert into reportFilterLists set reportID = :reportID,'
    .' column_name = :name, fs_von = :value');
if ($stmt === false){
    http_response_code(425);
    echo "\nFehler bei prepare Insert Filter-Lists\n";
    echo "\n".$dbh->errorInfo()[2];
    error_log("Fehler bei prepare von Insert Filter-Lists\n");
    error_log($dbh->errorInfo()[2]);
    die();
}

for ($i = 0; $i < $query->repFilter->size(); $i++){
    $filter = $query->repFilter->getPos($i);
    if ($filter->fs_list != null && count($filter->fs_list) > 0){
        $pars = array(
            'reportID' => $query->reportID,
            'name' => $filter->getName(),
            'value' => ''
        );
        foreach ($filter->fs_list as $value) {
            $pars['value'] = $value;
            if (false === $stmt->execute($pars)){
                http_response_code(425);
                echo "\nFehler bei execute von Insert Filter-Lists\n";
                echo "\n".$stmt->errorInfo()[2];
                error_log("Fehler bei execute von Insert Filter-Lists\n");
                error_log($stmt->errorInfo()[2]);
                error_log(print_r($pars,true));
                die();
            }
        }
    }
}// Ende FilterList

// ADD-Filter speichern
if ($query->loadSaveAddFilter && $query->addFilter->size() > 0){
    /**
     * 
     * @var RepFilter $af0
     */
    $af0 = $query->addFilter->getPos(0);
    $af0->deleteStatement = str_replace('reportFilters', 'reportAddFilters', $af0->deleteStatement);
    $af0->insertStatement = str_replace('reportFilters', 'reportAddFilters', $af0->insertStatement);
    $query->addFilter->save($query->reportID);
    
    // Filter-Listen fehlen noch
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'delete from reportAddFilterLists';
    $sql->addSel('reportID = ',$query->reportID);
    
    if ($sql->execute() === false){
        http_response_code(425);
        echo "\nFehler bei Delete Add-Filter-Lists";
        echo "\n".$sql->errorInfo()[2];
        die();
    }
        $stmt = $dbh->prepare('insert into reportAddFilterLists set reportID = :reportID,'
            .'  column_name = :name, fs_von = :value');
        if ($stmt === false){
            http_response_code(425);
            echo "\nFehler bei prepare Insert Add-Filter-Lists\n";
            echo "\n".$dbh->errorInfo()[2];
            error_log("Fehler bei prepare von Insert Add-Filter-Lists\n");
            error_log($dbh->errorInfo()[2]);
            die();
        }
        
        for ($i = 0; $i < $query->addFilter->size(); $i++){
            $filter = $query->addFilter->getPos($i);
            if ($filter->fs_list != null && count($filter->fs_list) > 0){
                $pars = array(
                'reportID' => $query->reportID,
                'name' => $filter->getName(),
                'value' => ''
                );
                foreach ($filter->fs_list as $value) {
                    $pars['value'] = $value;
                    if (false === $stmt->execute($pars)){
                        http_response_code(425);
                        echo "\nFehler bei execute von Insert Add-Filter-Lists\n";
                        echo "\n".$stmt->errorInfo()[2];
                        error_log("Fehler bei execute von Insert Add-Filter-Lists\n");
                        error_log($stmt->errorInfo()[2]);
                        error_log(print_r($pars,true));
                        die();
                    }
                }
            }
        }// Ende AddFilterList
    
            
    } // ENDE AddFilter speichern
        

// Speichern der JoinedTables
if ($query->repTables->size() > 0){
    $statement = $query->repTables->getPos(0)->deleteStatement;
    $stmt = $dbh->prepare($statement);
    if ($stmt === false){
        http_response_code(425);
        echo "\nFehler bei prepare von \n".$statement;
        echo "\n".$dbh->errorInfo()[2];
        error_log("Fehler bei prepare von \n".$statement);
        error_log($dbh->errorInfo()[2]);
        die();
    }
    if (false === $stmt->execute(array('reportID' => $query->reportID))){
        http_response_code(425);
        echo "\nFehler bei execute von \n".$statement;
        echo "\n".$stmt->errorInfo()[2];
        error_log("Fehler bei execute von \n".$statement);
        error_log($stmt->errorInfo()[2]);
        die();
    }
    $statement = $query->repTables->getPos(0)->insertStatement;
    $stmt = $dbh->prepare($statement);
    if ($stmt === false){
        http_response_code(425);
        echo "\nFehler bei prepare von \n".$statement;
        echo "\n".$dbh->errorInfo()[2];
        error_log("Fehler bei prepare von \n".$statement);
        error_log($dbh->errorInfo()[2]);
        die();
    }
    
    for ($i=0; $i < $query->repTables->size(); $i++){
        $pars = $query->repTables->getPos($i)->getInsertArray($query);
        $pars['reportID'] = $query->reportID;
        if (false === $stmt->execute($pars)){
            http_response_code(425);
            echo "\nFehler bei execute von \n".$statement;
            echo "\n".$stmt->errorInfo()[2];
            error_log("Fehler bei execute von \n".$statement);
            error_log($stmt->errorInfo()[2]);
            error_log(print_r($pars,true));
            die();
        }
    }
} // ENDE Speichern Joined Tables


// Log für Speichern
$logSql = new SqlAbfrage('WSR');
$logSql->sql = 'update wsrAdminReport set timeStored = sysdate()';
$logSql->addKlausel(', userStored = ', $_SESSION['Name']);
if ($query->reportImported){
    // Log für Importierten Eintrag
    if (isValid($query->importTimeCreated))
        $logSql->addKlausel(', timeCreated = ',$query->importTimeCreated);
    if (isValid($query->importUserCreated))
        $logSql->addKlausel(', userCreated = ',$query->importUserCreated);
    $logSql->addKlausel(', timeReleased = ',$query->importTimeReleased);
    $logSql->addKlausel(', userReleased = ',$query->importUserReleased);
}
$logSql->addSel('reportID = ', $query->reportID);
$erg = $logSql->executeE(false);
if ($erg === false || $erg->rowCount() < 1 ){
    error_log('Fehler beim Update wsrAdminReport: '. $logSql->errorInfo()[2]);
}


$query->reportImported = false;


echo "Die Report-Definition ".$query->title." wurde gespeichert.";
return;

