<?php
/**
 * Projekt: WS-Reporter
 *          Übersetzungen eintragen oder löschen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    5.8.2022
 *
 * Parameter:
 *      action: 
 *      langugage: Sprache aus der Spezifikation
 *      tLanguage: Sprache, in die übersetzt wird
 *      type: qLanguageType: Typ der Übersetzung
 *      text: Text, der übersetzt wird
 *      translation: Übersetzung
 *      
 * Es werden grundsätzlich decodierte Texte gespeichert, mit < statt &lt; usw.
 * Führende und folgende Zeilenvorschübe und Blanks werden gelöscht!    
 * CR (\r) werden entfernt.
 * 
 * Ausnahme: Kommentare. Sie werden ohnehin separat, d.h. ohne Bezug auf eine
 *           andere Sprache gespeichert.
 * 
 * Fehler-Code: 425
 */


/* Zuerst language und tLanguage prüfen */
require_once './includes/funktionen.inc';
anmeldungPruefen();



if (session_status() == PHP_SESSION_NONE) session_start();
require_once './phpsec/createDBH.inc';
global $dbh;
require_once './includes/classes.inc';
 
if (array_key_exists('language', $_REQUEST)){
    switch ($_REQUEST['language']) {
        case 'de':
         $language = 'de';
        break;
        case 'en':
            $language = 'en';
            break;
        case 'fr':
            $language = 'fr';
            break;
            
        default:
            http_response_code(400);
            echo 'Parameter language '.$_REQUEST['language'].'ist ungültig.';
            die();
        break;
    }
    
}else{
    http_response_code(400);
    echo 'Parameter language fehlt.';
    die();
}



if (!array_key_exists('action', $_REQUEST)){
    http_response_code(400);
    echo 'Parameter action fehlt.';
    die();
}

if (!array_key_exists('text', $_REQUEST)){
    http_response_code(400);
    echo 'Parameter text fehlt.';
    die();
}
$text = html_entity_decode($_REQUEST['text']);
$text = trim(str_replace("\r", '',$text));

switch ($_REQUEST['action']) {
    case 'setRText':
        if (!array_key_exists('type', $_REQUEST)){
            http_response_code(400);
            echo 'Parameter type fehlt.';
            die();
        }
        if (!array_key_exists('reportID', $_REQUEST)){
            http_response_code(400);
            echo 'Parameter reportID fehlt.';
            die();
        }
        
        $reportID = $_REQUEST['reportID'];
        switch ($_REQUEST['type']) {
            case qTranslate::REPORT_TITLE:
                error_log('Report Title übersetzen.');
                $sql = new SqlAbfrage('WSR');
                $sql->sql = 'update reportTitle set';
                $sql->addKlausel('reportTitle_'.$language.' = ', $text);
                $sql->addSel('reportID = ', $_REQUEST['reportID']);
                if (false === $sql->execute()){
                    echo 'FEHLER: '.$sql->errorInfo()[2];
                }
            break;
            case qTranslate::REPORT_COMMENT:
                $table = 'reportComment';
                error_log('Report Kommentar eintragen.');
                $inhalt = 'Kommentar';
            case qTranslate::REPORT_HINTS:
                if ($_REQUEST['type'] == qTranslate::REPORT_HINTS){
                    $table = 'reportHints';
                    error_log('Report techn. Hinweise eintragen.');                    
                    $inhalt = 'techn. Hinweise';
                }                
                $sql = new SqlAbfrage('WSR');
                $sql->sql = 'delete from '.$table;
                $sql->addSel('reportID = ', $reportID);
                $sql->addSel('rLanguage = ', $_REQUEST['language']);
                if (false === $sql->execute()){
                    echo 'FEHLER: '.$sql->errorInfo()[2];
                    die();
                }
                $sortNr = 0;
                $stmt = $dbh->prepare('insert into '.$table.' set reportID = :reportID,'
                    .' SortNr = :SortNr, rLanguage = :rLanguage, kommentar = :kommentar');
                if ($stmt === false){
                    http_response_code(425);
                    echo "\nFehler bei prepare Insert $inhalt\n";
                    echo "\n".$dbh->errorInfo()[2];
                    error_log("Fehler bei prepare von Insert $inhalt\n");
                    error_log($dbh->errorInfo()[2]);
                    die();
                }
                $kommentar = $_REQUEST['text'];
                $k = explode("\n",trim(str_replace("\r", '',$kommentar)));
                foreach ($k as $kom) {
                    $sortNr ++;
                    while (strlen($kom) > 248){
                        $pars = array(
                            'reportID' => $reportID,
                            'SortNr' => $sortNr,
                            'rLanguage' =>  $_REQUEST['language'],
                            'kommentar' => substr($kom,0,248)
                        );
                        $kom = substr($kom,248);
                        if (false === $stmt->execute($pars)){
                            http_response_code(425);
                            echo "\nFehler bei execute von Insert $inhalt\n";
                            echo "\n".$stmt->errorInfo()[2];
                            error_log("Fehler bei execute von Insert $inhalt\n");
                            error_log($stmt->errorInfo()[2]);
                            error_log(print_r($pars,true));
                            die();
                        }
                        $sortNr ++;
                    }
                    $pars = array(
                        'reportID' => $reportID,
                        'SortNr' => $sortNr,
                        'rLanguage' =>  $_REQUEST['language'],
                        'kommentar' => $kom."\n"
                    );
                    if (false === $stmt->execute($pars)){
                        http_response_code(425);
                        echo "\nFehler bei execute von Insert $inhalt\n";
                        echo "\n".$stmt->errorInfo()[2];
                        error_log("Fehler bei execute von Insert $inhalt\n");
                        error_log($stmt->errorInfo()[2]);
                        error_log(print_r($pars,true));
                        die();
                    }
                }
            break;
            default:
                ;
            break;
        }
        break;
    case 'setQText':
        /*
        if (!array_key_exists('type', $_REQUEST)){
            http_response_code(400);
            echo 'Parameter type fehlt.';
            die();
        }
        */
        if (!array_key_exists('table_name', $_REQUEST)){
            http_response_code(400);
            echo 'Parameter table_name fehlt.';
            die();
        }
        $table = $_REQUEST['table_name'];
        
        if ($_REQUEST['type'] == qTranslate::QUERY_COMMENT){
            error_log('Query->table_description eintragen.');
            $targettable = 'queryComment';
            $inhalt = 'Kommentar';
        }else
            if ($_REQUEST['type'] == qTranslate::QUERY_HINTS){
                error_log('Query->table_hints eintragen.');
                $targettable = 'queryHints';
                $inhalt = 'techn. Hinweise';
        }else{
            http_response_code(425);
            echo "FEHLER: Ungültiger Typ für Eintrag ".$_REQUEST['type'];
            die();
        }
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'delete from '.$targettable;
        $sql->addSel('table_name = ', $table);
        $sql->addSel('qLanguage = ', $language);
        if (false === $sql->execute()){
            echo 'FEHLER: '.$sql->errorInfo()[2];
            die();
        }
        if (! isValid($_REQUEST['text'])) return;
        $sortNr = 0;
        $stmt = $dbh->prepare('insert into '.$targettable.' set table_name = :table_name,'
            .' SortNr = :SortNr, qLanguage = :qLanguage, kommentar = :kommentar');
        if ($stmt === false){
            http_response_code(425);
            echo "\nFehler bei prepare Insert $inhalt\n";
            echo "\n".$dbh->errorInfo()[2];
            error_log("Fehler bei prepare von Insert $inhalt\n");
            error_log($dbh->errorInfo()[2]);
            die();
        }
        
        $k = explode("\n",trim(str_replace("\r", '', $_REQUEST['text'])));
            foreach ($k as $kom) {
                $sortNr ++;
                while (strlen($kom) > 248){
                    $pars = array(
                        'table_name' => $table,
                        'SortNr' => $sortNr,
                        'qLanguage' => $language,
                        'kommentar' => substr($kom,0,248)
                    );
                    $kom = substr($kom,248);
                    if (false === $stmt->execute($pars)){
                        http_response_code(425);
                        echo "\nFehler bei execute von Insert $inhalt\n";
                        echo "\n".$stmt->errorInfo()[2];
                        error_log("Fehler bei execute von Insert $inhalt\n");
                        error_log($stmt->errorInfo()[2]);
                        error_log(print_r($pars,true));
                        die();
                    }
                    $sortNr ++;
                }
                $pars = array(
                    'table_name' => $table,
                    'SortNr' => $sortNr,
                    'qLanguage' => $language,
                    'kommentar' => $kom."\n"
                );
                if (false === $stmt->execute($pars)){
                    http_response_code(425);
                    echo "\nFehler bei execute von Insert $inhalt\n";
                    echo "\n".$stmt->errorInfo()[2];
                    error_log("Fehler bei execute von Insert $inhalt\n");
                    error_log($stmt->errorInfo()[2]);
                    error_log(print_r($pars,true));
                    die();
                }
            }
        break;
    case 'enter':
        if (array_key_exists('tLanguage', $_REQUEST)){
            switch ($_REQUEST['tLanguage']) {
                case 'de':
                    $tLanguage = 'de';
                    break;
                case 'en':
                    $tLanguage = 'en';
                    break;
                case 'fr':
                    $tLanguage = 'fr';
                    break;
                    
                default:
                    http_response_code(400);
                    echo 'Parameter tLanguage '.$_REQUEST['tLanguage'].'ist ungültig.';
                    die();
                    break;
            }
            
        }else{
            http_response_code(400);
            echo 'Parameter tLanguage fehlt.';
            die();
        }
        
        if ($language == $tLanguage){
            http_response_code(400);
            echo 'Parameter language und tLanguage sind gleich.';
            die();
        }if (!array_key_exists('type', $_REQUEST)){
            http_response_code(400);
            echo 'Parameter type fehlt.';
            die();
        }
        if (!array_key_exists('translation', $_REQUEST)){
            http_response_code(400);
            echo 'Parameter translation fehlt.';
            die();
        }
        $translation = html_entity_decode($_REQUEST['translation']);
        $translation = trim(str_replace("\r", '',$translation));
        
        $sql = new SqlAbfrage('WSR');
        $sql->sql = 'select '.$tLanguage.' as t from queryTranslations';
        $sql->addSel($language.' = ',$text);
        $sql->addSel('qLanguageType = ',$_REQUEST['type']);
        $stmt = $sql->execute();
        if ($stmt === false) die();
        if ($z = $stmt->fetchObject()){
            if ($z->t != $translation){
                $sql = new SqlAbfrage('WSR');
                $sql->sql = 'update queryTranslations set';
                $sql->addKlausel($tLanguage.' = ',$translation);
                $sql->addSel($language.' = ',$text);
                $sql->addSel('qLanguageType = ',$_REQUEST['type']);
                if (!$sql->execute()) {
                    http_response_code(425);
                    echo $sql->errorInfo()[2];
                    die();
                }
            }
        }else{
            $sql = new SqlAbfrage('WSR');
            $sql->sql = 'insert into queryTranslations set';
            $sql->addKlausel($tLanguage.' = ',$translation);
            $sql->addKlausel(', '.$language.' = ',$text);
            $sql->addKlausel(', qLanguageType = ',$_REQUEST['type']);
            if (!$sql->execute()) {
                http_response_code(425);
                echo $sql->errorInfo()[2];
                die();
            }
        }
    ;
    break;

    
    
    default:
        http_response_code(400);
        echo 'Parameter action '.$_REQUEST['action'].'ist ungültig.';
        die();
        break;
}

