<?php 
/**
 * Projekt: WS-Reporter
 *          Übersetzungen zu den Report-Definitionen
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    12.8.2022
 * 
 * Hinweise:
 * Es wird die gespeicherte Version eines Reports übersetzt.
 * Das aktuelle Query-Objekt aus der Session wird ignoriert.  
 *
 */

/* ??
if (array_key_exists('class_table', $_REQUEST)){
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 
}
*/
include 'includes/defender.inc';

require_once './includes/funktionen.inc';
require_once './includes/classes.inc';
require_once './includes/qTranslate.inc';
if (session_status() == PHP_SESSION_NONE)  session_start();
anmeldungPruefen();
uebersetzerRechteVerlangen();

/**
 * Frage: Soll hier das aktuelle Objekt übersetzt werden, indem 
 * man die einzelnen Texte hervorholt?
 * 
 * Oder wird die Übersetzungstabelle zur Bearbeitung angezeigt und
 * die nicht übersetzten Einträge folgen unten?
 * 
 * Können nur gespeicherte Datenquellen übersetzt werden?
 * Was ist beim Import?
 *  
 * Antworten:
 * * Übersetzte Einträge sollten von nicht übersetzten Einträgen separiert werden. 
 * * Gruppierung nach Art des Eintrags (qLanguageType)
 * 
 * Überlegungen:
 * * Wenn nur gespeicherte Datenquellen übersetzt werden können,
 *   kann die Aufbereitung der Tabelle in der Datenbank mit 
 *   Temporary Tables erfolgen.
 */

$sprache = 'de';
if (array_key_exists('Sprache', $_SESSION)) $sprache = substr($_SESSION['Sprache'], 0,2);

if (array_key_exists('reportID', $_REQUEST)){
    $reportID = $_REQUEST['reportID'];
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select reportVersion, reportTitle_'.$sprache.' as title, '
        . 'reportTitle_de, reportTitle_en, reportTitle_fr, reportID from reportTitle';
    $sql->addSel('reportID = ', $reportID);
    $stmt = $sql->execute();
    error_log('Report über ID ausgewählt');
}elseif (array_key_exists('report', $_REQUEST)){
    $title = $_REQUEST['report'];
    $sql = new SqlAbfrage('WSR');
    $sql->sql = 'select reportVersion, reportTitle_'.$sprache.' as title, '
        . 'reportTitle_de, reportTitle_en, reportTitle_fr, reportID from reportTitle';
    $sql->addSel('reportTitle_'.$sprache.' = ', $title);
    $stmt = $sql->execute();
    error_log('WARNUNG: Report über Titel ausgewählt');
}else{
    http_response_code(400);
    echo WSRTranslate::translate('Name und Nummer des Reports fehlen.');
    die();
}
if ($stmt === false){
    http_response_code(400);
    echo 'Fehler bei Fetch Report Title';
    echo $sql->errorInfo()[2];
    die();    
}
$z = $stmt->fetch(PDO::FETCH_ASSOC);
if ($z === false){
    http_response_code(400);
    echo WSRTranslate::translate('Fehler: Report nicht gefunden.');
    die();
}
$reportID = $z['reportID'];
$title = $z['title'];
$version = $z['reportVersion'];
if ($version < 2){
    http_response_code(400);
    echo WSRTranslate::translate('Fehler: Nur aktuelle Versionen können übersetzt werden.').'<br>';
    echo WSRTranslate::translate('Der Report muss vorher neu gespeichert werden.');
    die();
}



/* Attribute einlesen */
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select * from reportAttributes';
$sql->addSel('reportID = ', $reportID);
$stmt = $sql->execute();
if ($stmt === false) {
    var_dump($sql);
    var_dump($dbh->errorInfo());
    die();
}
$zr = $stmt->fetchObject();
$rLanguage = $zr->rLanguage;
$rTranslations = explode(' ', trim($zr->rTranslations));
if (count($rTranslations) < 1 || !isValid($rTranslations[0])){
    http_response_code(400);
    echo WSRTranslate::translate('Fehler: Zur Übersetzung sind keine Sprachen deklariert.');
//    var_dump($rLanguage);var_dump($rTranslations);
    die();
}
error_log('Sprachen: '.$rLanguage.' '.print_r($rTranslations,true));

// Komentare einlesen
$comments = array('de'=>'', 'en'=>'', 'fr'=>'');
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select SortNr, rLanguage, kommentar from reportComment';
$sql->addSel('reportID = ', $reportID);
$sql->sql .= ' order by SortNr';
$stmt = $sql->execute();
if ($stmt === false){
    echo "<pre>Fehler beim Einlesen der Kommentare\n";
    echo $sql->errorInfo()[2];
    die();
}
while ($zc = $stmt->fetchObject()){
    $comments[$zc->rLanguage] .= $zc->kommentar;
}
$stmt->closeCursor();

// Technische Hinweise einlesen
$hints = array('de'=>'', 'en'=>'', 'fr'=>'');
$sql = new SqlAbfrage('WSR');
$sql->sql = 'select SortNr, rLanguage, kommentar from reportHints';
$sql->addSel('reportID = ', $reportID);
$sql->sql .= ' order by SortNr';
$stmt = $sql->execute();
if ($stmt === false){
    echo "<pre>Fehler beim Einlesen der Technischen Hinweise\n";
    echo $sql->errorInfo()[2];
    die();
}
while ($zc = $stmt->fetchObject()){
    $hints[$zc->rLanguage] .= $zc->kommentar;
}
$stmt->closeCursor();

/* Liste der zu übersetzenden Texte */
$sql = new SqlAbfrage('WSR');

$sql->sql = "select distinct case when t.qLanguageType is null then 2 else 1 end as gsort, q.type, q.text, t.*  from (";

$sql->sql .= "select 1 as sort, ".qTranslate::QT_LABEL." as type, prefix as text from reportJoinedTables";
$sql->addSel('reportID = ', $reportID);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, web_title as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, web_link_title as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type, web_link_value as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type, web_link as text from reportRows";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->addWhere('web_link like "#%"');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, label as text from reportValues";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, column_label as text from reportColumns";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_WEBLINK." as type, web_link as text from reportColumns";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);
$sql->addWhere('web_link like "#%"');

$sql->sql .= "\nunion\nselect 1 as sort, ".qTranslate::QT_LABEL." as type, web_title as text from reportColumns";
$sql->firstWhere = true;
$sql->addSel('reportID = ', $reportID);


$sql->sql .="\n) q left join queryTranslations t on t.".$rLanguage." = trim(q.text) "; // and q.type = t.qLanguageType";        
$sql->sql .="\n and (q.type != ".qTranslate::QT_SQLTABLE.' or t.qLanguageType = '.qTranslate::QT_SQLTABLE.')'; 

$sql->sql .="\nwhere q.text is not null and q.text != ''";
$sql->sql .="\norder by gsort,  type, q.text";

$stmt = $sql->execute();
if ($stmt === false){
    echo '<pre>';
    var_dump($sql);
    var_dump($dbh->errorInfo());
    die();
}

?>
<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<title>WS-Reporter: <?= WSRTranslate::translate('Übersetzungen zum Report').' '.$title ?></title>
<meta name="GENERATOR" content="WS-PHP: editQuery.php">
<link rel="stylesheet" href="./jscss/standard2.css">
<link rel="stylesheet" href="./jscss/WSReport.css" >
<script type="text/javascript" src="./jscss/translate.js">
	</script>
<script type="text/javascript" src="./jscss/tableHeader.js">
	</script>
<link rel="stylesheet" href="./jscss/tableFolder.css">
<script src="./jscss/tableFolder.js">
// Wird unten aktiviert, nachdem die Rowspans gesetzt sind
	</script>	
<script type="text/javascript">
 language = "<?=$rLanguage?>"; 
	</script>
	
</head>
<?php 
if ($_SERVER["HTTP_HOST"] == "localhost:8898"){
    ?>
<body class = "entwicklung">	
<?php
} else echo '<body>';
?>

<h1><?= WSRTranslate::translate('Übersetzungen zum Report').' '.$title ?></h1>

<div id="Editor" class = "editorBox">
 <div class = "xButton" onclick="closeEditor()" title="schließen">&otimes;</div>
 <h3><?=WSRTranslate::translateH('Verwendungen des Textes')?></h3>
<pre id="VerwText"></pre>
<?=WSRTranslate::translateH('wird verwendet von:')?><br>
<div id = "VerwAnzeige">
</div>
<br>  <button onclick="closeEditor()">Schließen</button> 
</div>

<table id='Tabelle'>
<thead>
<tr><th class="WSFHeader">&nbsp;</th><th class='fts'>Text (<?=$rLanguage ?>)</th>
<?php 
foreach ($rTranslations as $trans) {
    echo '<th>'.$trans.'</th>';
}
$rst = 'colspan = "'.(count($rTranslations)+1).'"';
?>
</tr>
</thead>
<tbody>
<tr><th>&nbsp;</th><th <?=$rst ?>><?=WSRTranslate::translate('Report-Titel') ?></th></tr>
<tr><th>&nbsp;</th><td class='fts' ><textarea rows="2" cols="70" id='<?='T0'.$rLanguage?>' readonly>
<?=$z['reportTitle_'.$rLanguage]?>
</textarea></td>  
<?php 
foreach ($rTranslations as $trans) {
?>
<td><textarea rows="2" cols="70" id='<?="T0".$trans ?>' 
lang="<?=$trans?>" onchange='setRText(this,<?=$reportID.','.qTranslate::REPORT_TITLE?>)' >
<?= $z['reportTitle_'.$trans]?>
</textarea></td>
<?php 
}
?>
</tr>
<tr><th rowspan='2' class="WSFHeader">&nbsp;</th><th <?=$rst ?>><?=WSRTranslate::translate('Kommentar') ?></th></tr>
<tr></th><td class='fts' ><textarea rows="15" cols="70" id='<?='T1'.$rLanguage?>' readonly>
<?=$comments[$rLanguage]?>
</textarea></td>  
<?php 
foreach ($rTranslations as $trans) {
?>
<td><textarea rows="15" cols="70" id='<?="T1".$trans ?>' 
lang="<?=$trans?>" onchange='setRText(this,<?=$reportID.','.qTranslate::REPORT_COMMENT?>)' >
<?= $comments[$trans]?>
</textarea></td>
<?php 
}
?>
</tr>

<tr><th rowspan='2' class="WSFHeader">&nbsp;</th><th <?=$rst ?>><?=WSRTranslate::translate('Technische Hinweise') ?></th></tr>
<tr></th><td class='fts' ><textarea rows="15" cols="70" id='<?='T1'.$rLanguage?>' readonly>
<?=$hints[$rLanguage]?>
</textarea></td>  
<?php 
foreach ($rTranslations as $trans) {
?>
<td><textarea rows="15" cols="70" id='<?="T1".$trans ?>' 
lang="<?=$trans?>" onchange='setRText(this,<?=$reportID.','.qTranslate::REPORT_HINTS?>)' >
<?= $hints[$trans]?>
</textarea></td>
<?php 
}
?>
</tr>


<?php

$type = -1;
$gsort = 1;
$c = 5;
$rowspans = array();
if (! array_key_exists('SQL', $_REQUEST))
while ($z = $stmt->fetch(PDO::FETCH_ASSOC)){
    $c++;
    if ($z['gsort'] != $gsort){
        $gsort = $z['gsort'];
        $type = -1;
        echo '<tr><th>&nbsp;</th><th '.$rst.'>Ohne Übersetzungen</th></tr>';
    }
    if ($z['type'] != $type){
        $type = $z['type'];
        echo '<tr><th id="R'.$gsort.$type.'" class="WSFHeader">&nbsp;</th>';
        $rowspans['R'.$gsort.$type]  = 1;
        switch ($type) {
            case qTranslate::QT_LABEL:
                echo '<th '.$rst.'>'.WSRTranslate::translate('Label').'</th></tr>';
            break;
            case qTranslate::QT_SQL:
                echo '<th '.$rst.'>SQL</th></tr>';
                break;
            case qTranslate::QT_SQLDISP:
                echo '<th '.$rst.'>'.WSRTranslate::translate('Display-SQL').'</th></tr>';
                break;
            case qTranslate::QT_SQLTABLE:
                echo '<th '.$rst.'>'.WSRTranslate::translate('Tabellen').'</th></tr>';
                break;
            case qTranslate::QT_TEXT:
                echo '<th '.$rst.'>'.WSRTranslate::translate('Text').'</th></tr>';
                break;                
            case qTranslate::QT_WEBLINK:
                echo '<th '.$rst.'>'.WSRTranslate::translate('Web-Link oder Web-Link-Anzeige').'</th></tr>';
                break;
            default:
                ;
            break;
        }
    }
    $rowspans['R'.$gsort.$type]  ++;
?>
<tr><td class='fts nowrap' ><textarea rows="2" cols="70" id='<?='T'.$c.$rLanguage?>' readonly ondblclick = 'showRVars(this.value)'>
<?=htmlspecialchars($z['text'])?>
</textarea>
<span class="gButton"  title='<?=WSRTranslate::translate('Eintrag löschen')?>' onclick="deleteTranslationById('<?='T'.$c.$rLanguage?>','<?=$rLanguage?>')">DEL</span>
<span class="gButton" title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$c.$rLanguage?>')">V</span>
</td>  
<?php 
foreach ($rTranslations as $trans){
?>
<td class='nowrap'><textarea rows="2" cols="70" id='<?='T'.$c.$trans?>' 
lang="<?=$trans ?>" onchange='enter(this,<?=$type?>)' >
<?= htmlspecialchars($z[$trans])?>
</textarea>
<span class="gButton"  title='<?=WSRTranslate::translate('Verwendungen anzeigen')?>' onclick="showVarsById('<?='T'.$c.$trans?>')">V</span>
<span class="gButton"  title='<?=WSRTranslate::translate('Übersetzungen anzeigen')?>' onclick="showTranslationById('<?='T'.$c.$trans?>')">T</span>
</td>
<?php 
}
?>
</tr>
<?php 
}

?>
</tbody>
</table>
<p><b><?=WSRTranslate::translate('Hinweise')?>:</b><br>
<?=WSRTranslate::translate('Ein Klick auf V zeigt die Verwendung des Textes in Datenquellen und Reports.')?><br>
<?=WSRTranslate::translate('Wenn eine Übersetzung wegen doppelten Einträgen abgelehnt wird,'
    .' zeigt ein Klick auf T die Übersetzungen zu diesem Text.')?>
<?php
if ($c < 1 || array_key_exists('SQL', $_REQUEST)){
    $s = $sql->sql;
    for ($i = 0; $i <= 8; $i++){
        $s = str_replace(':filterPar'.$i, '"'.$table.'"', $s);
    }
    echo '<pre>'.$s.'</pre>';
}else{
?>
 <script>
    window.addEventListener('load', function() {
<?php    
    foreach ($rowspans as $id => $count){
        echo 'document.getElementById("'.$id.'").rowSpan='.$count.";\n";
    }
?>    
    var tf = new TableFolder('Tabelle',TableFolder.NachKlassen,TableFolder.Eingeklappt,true);
    }
   ); 
   </script>
<?php    
}
?>
</body>
</html>
